package com.iflytek.cloud;

import android.os.Bundle;


/**
 * <h4>身份验证回调接口</h4>
 * <script>gShowExtraIdentity();</script>
 * <p>通过实现此接口，获取当前身份验证的状态和结果</p>
 * 
 * @author <script>gShowAuthor();</script>
 * @since version:1073
 * @see IdentityResult
 * @see IdentityVerifier#startWorking
 */
public abstract interface IdentityListener {

	/**
	 * <h4>结果回调</h4>
	 * <p>
	 * 		一次会话的结果可能会多次返回（即多次回调此函数），通过参数2,判断是否是最后
	 * 一个结果，true时为最后一个结果，否则不是。当最后一个结果返回时，本次会话结束，
	 * 录音也会停止，在重新调用 {@link IdentityVerifier#startWorking}开启新的会话前，
	 * 停止调用{@link IdentityVerifier#writeData}写入数据，
	 * 当出现错误，或应用层调用{@link IdentityVerifier#cancel}取消当次识别时，在当次识
	 * 别会话过程可能不会回调此函数。
	 * </p>
	 * 
	 * @param result 验证结果，默认是未解析的Json结果，请参考{@link IdentityResult}
	 * @param islast 是否最后一次的结果
	 * @see IdentityResult
	 */
	void onResult(IdentityResult result, boolean islast);

	/**
	 * <h4>错误回调</h4>
	 * <p>
	 * 		当此函数回调时，说明当次会话出现错误，会话自动结束，再次调用
	 * {@link IdentityVerifier#startWorking}开启新的会话前，停止调用
	 * {@link IdentityVerifier#writeData}写入数据。
	 * </p>
	 * 
	 * @param error 错误类型，获取错误码信息请查看{@link SpeechError} 详细错误码请看{@link ErrorCode}
	 * 
	 */
	void onError(SpeechError error);
	
    /**
     * <h4>事件回调</h4>
     * <p>
	 * 		扩展用接口，由具体业务进行约定。
	 * </p>
	 * 
     * @param eventType 消息类型。
     * @param arg1 参数1，eventType为{@link SpeechEvent#EVENT_VOLUME}时，为音量值。
     * @param arg2 参数2。
     * @param obj 扩展参数。
     */
	void onEvent(int eventType, int arg1,int arg2, Bundle obj);
	
}
