package com.iflytek.cloud;

import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;

import org.json.JSONException;
import org.json.JSONObject;

import android.app.ActivityManager;
import android.content.BroadcastReceiver;
import android.content.ContentResolver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.pm.PackageManager.NameNotFoundException;
import android.content.pm.ResolveInfo;
import android.database.Cursor;
import android.net.Uri;
import android.text.TextUtils;
import android.util.Base64;
import android.util.Log;

import com.iflytek.cloud.msc.module.SpeechInterface;
import com.iflytek.cloud.msc.msp.ComSession;
import com.iflytek.cloud.msc.msp.MscCommon;
import com.iflytek.cloud.msc.util.DataUtil;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.param.HashParam;
import com.iflytek.cloud.param.MscKeys;

/*import com.iflytek.idata.extension.IFlyCollectorExt;*/
//import com.iflytek.idata.extension.IFlyCollectorExt;
import com.iflytek.msc.MSC;
import com.iflytek.msc.MSCSessionInfo;
import com.iflytek.speech.SpeechComponent;
import com.iflytek.speech.UtilityConfig;

/**
 * <h3>SDK配置类</h3>
 * <p>
 * 		本类为SDK入口类，除{@link Setting}、{@link Version}类外，第一次使用任意一个其
 * 他的业务类前，都至少创建一次本类的实例，以初始化SDK。
 * </p>
 * <p>
 * 		本类使用单例，调用者使用本类的对象，只需要通过{@link #createUtility}创建
 * 一次对象后，便可一直使用该对象，直到通过调用{@link #destroy()}进行单例对象销毁。调
 * 用者可通过{@link #getUtility()}获取当前已经创建的单例。在销毁本类的单例对象后，
 * 需要先通过{@link #createUtility}再次创建单例对象，方可再使用。
 * </p>
 * 
 * @since version:1073
 * @author <script>gShowAuthor();</script>
 * @see SpeechRecognizer
 * @see SpeechSynthesizer
 * @see SpeechUnderstander
 * @see TextUnderstander
 * @see SpeakerVerifier
 */
public class SpeechUtility extends SpeechInterface{
	private static final String TAG = "SpeechUtility";
	// 用户语音配置对象
	private static SpeechUtility sUtility = null;
	
	// 如果是注册用户，需要设置用户名，密码参数
	private static final String KEY_USR = "usr";
	private static final String KEY_PWD = "pwd";
	
	/** 所有组件信息 */
	private ArrayList<SpeechComponent> mAllComponents = new ArrayList<SpeechComponent>();

	// 讯飞语记的版本号
	private int mVersionCode = -1;
	
	/** 上下文环境变量 */
	private Context mContext = null;
	
	/** 服务是否安装 */
	private boolean serviceInstalled = false;
	
	/* 语记安装与卸载的广播类 */
	private AppBroadcastReceiver mAppBroadcastReceiver = null;
	
	/* 通过编译脚本选项，在编译时自动修改，在V5版本时默认为AUTO，V5+版本时默认为MSC */
	/**默认引擎模式*/
	final public static ENGINE_MODE DEF_ENGINE_MODE = ENGINE_MODE.MSC;
	protected ENGINE_MODE engineMode = DEF_ENGINE_MODE;
	
	
	/**本地资源信息JSON格式的字符串类型**/
	public static final String TAG_RESOURCE_CONTENT = "tag_rescontent";
	/**本地资源信息JSON返回的code**/
	public static final String TAG_RESOURCE_RET = "ret";
	/**本地资源信息JSON返回的result**/
	public static final String TAG_RESOURCE_RESULT = "result";
	/**本地资源contentprovider访问的URI**/
	private static final String LOCAL_RESOURCE_URI = "content://com.iflytek.vflynote.providers.LocalResourceProvider";
	/**getParameter 方法开始支持的版本 **/
	private static final int GETPARAMETER_SUPPORTED_VER = 97;
	/**getParameter 方法不支持厂商版本 **/
	private static final int GETPARAMETER_NOT_SUPPORTED_LOWER = 10000;
	private static final int GETPARAMETER_NOT_SUPPORTED_UPPER = 11000;
	
	/**
	 * <h4>创建配置单例</h4>
	 * <p>
	 * 		创建配置单例，同时，也是初始化SDK的操作。
	 * </p>
	 * </p>
	 * 		注册应用请前往开发者网站，<script>gShowDeveloperWebsite();</script>。
	 * </p>
	 * 
	 * @param params  
	 * 			启动参数，包含以下参数：<br>
	 * 			{@link SpeechConstant#APPID}，必须，如：12345678；<br>
	 * 			server_url，可选，如：<script>gShowServerurl();</script>；<br>
	 * 			{@link SpeechConstant#LIB_NAME}，可选，如：mylibname；<br>
	 * 			各参数间，以英文逗号分隔，如"appid=12345678,server_url=
	 * 			<script>gShowServerurl();</script>,lib_name=mylibname"。
	 * @return 配置对象
	 */
	public static synchronized SpeechUtility createUtility(Context context,String params) {
		//DebugLog.LogD("SpeechUtility createUtility");		
		
		synchronized( sSync ){
			if(sUtility == null){
				//根据参数来判断是否进行进程检查
				HashParam param = new HashParam();
				param.putMultiParam(params);
				if( !param.getBoolean(SpeechConstant.FORCE_LOGIN, false) && !isOwnProcess(context.getApplicationContext())){
					DebugLog.LogE("init failed, please call this method in your main process!");
					sUtility = null;
				}else {
					try{
						sUtility = new SpeechUtility(context,params);
					}catch( SpeechError e ){
						DebugLog.LogE("init failed" );
						DebugLog.LogE( e );
					}//end of try-catch
				}
			}
		}//end of sSync
			
		return sUtility;
	}
	
	/**
	 * <h4>获取配置单例</h4>
	 * <p>
	 * 		通过函数获取已创建的单例对象。当单例对象未创建时，将返回null，此时应先通过
	 * {@link #createUtility}创建单例对象。
	 * </p>
	 * @return 配置单例
	 * 
	 * @see #createUtility
	 * @see #destroy()
	 */
	public static synchronized SpeechUtility getUtility(){
		return sUtility;
	}
	
	/**
	 * 构造函数
	 */
	private SpeechUtility(Context context,String params) throws SpeechError {
		super();
		//DebugLog.LogD("SpeechUtility SpeechUtility construct");
		mContext = context.getApplicationContext();
		super.setParameter(SpeechConstant.PARAMS, params);
		
		// 加载底层库
		String libName =  mSessionParams.getString(SpeechConstant.LIB_NAME, "msc");
		MSC.loadLibrary(libName);
		// 设置jni中是否显示log
		DebugLog.updateJniLogStatus();
		// 设置参数列表
		setParameter(SpeechConstant.PARAMS,params);
		// @ { 根据云计算研究院Jira 需求，从msc 中拿掉语记，因此默认以MSC 引擎
		engineMode = ENGINE_MODE.MSC;
		//灵犀云下的默认引擎模式为MSC
		/*if( !MSC.isIflyVersion() ){
			engineMode = ENGINE_MODE.MSC;
		}
		
		// 引擎选择模式
		String mode = getParameter(SpeechConstant.ENGINE_MODE);
		if(SpeechConstant.MODE_MSC.equals(mode))
			engineMode = ENGINE_MODE.MSC;
		else if(SpeechConstant.MODE_PLUS.equals(mode))
			engineMode = ENGINE_MODE.PLUS;*/
		//@ 需求结束}

		// msc动态库已加载的情况下，进行MSC初始化操作
		final int lgiError = msplogin();
		if( ErrorCode.SUCCESS != lgiError ){
			throw new SpeechError(lgiError);
		}
		initComponentInfo();

		//registerBroadcast();
		String appid =  this.getParameter(SpeechConstant.APPID);
		try{
			//UserLogger userlog = UserLogger.getUserLogger(context);
			// 统计分析，策略更新
			//userlog.updatePolicyProcess();
			// 统计分析，收集发送
			//userlog.collectAndUploadProcess();
			//添加统计分析模块，header信息单独获取,
			//添加开关配置，防止手机厂商用户检测流量数据
			String enableCollect = this.getParameter("enable_sunflower");
			/*if (TextUtils.isEmpty(enableCollect) || !enableCollect.equals("false")) {
			//	Logger.setDebugMode(false);
                collectorThread thread = new collectorThread();
                thread.setAppid(appid);
                thread.setContext(context.getApplicationContext());
                thread.start();
			}*/
		}catch (Exception e) {
			DebugLog.LogW("userLog creating failed");
		}
		String strDcEnable = getParameter( MscKeys.KEY_LXY_DATA_COLLECT_ENABLE );
		boolean enable = true;
		if( !TextUtils.isEmpty(strDcEnable) && MscKeys.VAL_FALSE.equals(strDcEnable) ){
			enable = false;
		}
		DebugLog.LogD( "DC init enable="+strDcEnable );
	}
    /*class collectorThread extends Thread {
        String appid="";
        Context context;
        public void setAppid(String appid){
            this.appid = appid;
        }
        public void setContext(Context context){
            this.context =context;
        }
        public void run() {
            IFlyCollectorExt instance = IFlyCollectorExt.getInstance(context.getApplicationContext());
            //msc在DaaS平台的appid
            instance.setAppId("5d2386c983");
            //应用在msc定义的appid，即子id
            instance.setSubId(appid);
        }
    };*/
	/**
	 * 初始化操作
	 * @return 初始化返回值
	 */
	private int msplogin() 
	{
		if(!MSC.isLoaded())
			return ErrorCode.ERROR_ENGINE_NOT_SUPPORTED;
		DebugLog.LogD("SpeechUtility start login");		
		MscCommon mscer = new MscCommon(mContext,mSessionParams);
		SpeechError error = ((MscCommon)mscer).login(mSessionParams.getString(KEY_USR),mSessionParams.getString(KEY_PWD));
		if(error == null)
			return ErrorCode.SUCCESS;
		else
			return error.getErrorCode();
	}
	
	/**
	 * 注销操作
	 * @return 注销是否成功
	 */
	private boolean msplogout() {
		boolean ret = true;
		
		if( MSC.isLoaded() ){
			ret = ComSession.Logout();
		}
		
		return ret;
	}
	
	
	/**
	 * <h4>销毁单例对象</h4>
	 * <p>
	 * 		通过本函数，销毁由{@link #createUtility}创建的单例对象，同时也是逆
	 * 初始化SDK。
	 * </p>
	 * <p>
	 * 		在调用本函数进行销毁前，应先保证所有业务的会话均已结束，否则可能引起在
	 * 当前应用的生命周期内，无法再使用SDK进行业务会话的问题。
	 * </p>
	 * <p>
	 * 		在调用本函数成功销毁对象后，所有业务会话在下一次通过{@link #createUtility}
	 * 创建实例前，将不能再使用。
	 * </p>
	 * 
	 * @return 销毁成功：true；销毁失败：false。 
	 * @see #createUtility
	 * @see #getUtility()
	 */
	public boolean destroy() {
		boolean ret = true;
		
		final SpeechUtility utility = sUtility;
		if(utility != null){
			super.destroy();
			ret = msplogout();
		}
		
		if( ret ){
			final AppBroadcastReceiver receiver = this.mAppBroadcastReceiver;
			if(null != receiver){
				mContext.unregisterReceiver(receiver);
			}
			mAppBroadcastReceiver = null;
			
			DebugLog.LogD( "SpeechUtility destory success" );	
			
			//消毁数据采集
		/*	DataCollect.destory();*/
			
			synchronized( sSync ){
				sUtility = null;
			}//end of sync
		}
		
		return ret;
	}
	
	/*
	 * 返回语记是否安装的状态值，如果重新检测，需要销毁SpeechUtility后重新创建
	 * @return
	 */
	protected boolean isServiceInstalled()
	{
		boolean isInstalled = false;
		try {
			PackageInfo packageInfo = mContext.getPackageManager().getPackageInfo(UtilityConfig.COMPONENT_PKG, 0);
			if(null != packageInfo)
				isInstalled = true;
		} catch (NameNotFoundException e) {
		}
		
		return isInstalled;
	}
	
	/**
	 * 初始化组件信息
	 */
	private void initComponentInfo() {
		
		if(checkServiceInstalled()){
			// 识别
			getComponentInfo(UtilityConfig.ACTION_SPEECH_RECOGNIZER);
			// 合成
			getComponentInfo(UtilityConfig.ACTION_SPEECH_SYNTHESIZER);
			// 语音语义
			getComponentInfo(UtilityConfig.ACTION_SPEECH_UNDERSTANDER);
			// 文本语义
			getComponentInfo(UtilityConfig.ACTION_TEXT_UNDERSTANDER);
			//唤醒
			getComponentInfo(UtilityConfig.ACTION_SPEECH_WAKEUP);
		}
	}
	
	/*
	 * 讯飞语记安装与卸载的广播
	 */
	private void registerBroadcast(){
		// 注册广播，检查语记是否发生安装或卸载，如发生则需调用 checkServiceInstalled
		mAppBroadcastReceiver = new AppBroadcastReceiver();    
		IntentFilter intentFilter=new IntentFilter();    
		intentFilter.addAction(Intent.ACTION_PACKAGE_ADDED);
		intentFilter.addAction(Intent.ACTION_PACKAGE_REMOVED);
		intentFilter.addAction(Intent.ACTION_PACKAGE_REPLACED);
		intentFilter.addDataScheme("package");    
		mContext.registerReceiver(mAppBroadcastReceiver, intentFilter);
	}
	
	/**
	 * 广播继承类
	 */
	private class AppBroadcastReceiver extends BroadcastReceiver {    
		@Override   
		public void onReceive(Context context, Intent intent) {  
			String action=intent.getAction();    
			String packageName=intent.getDataString();  
			String plusPackageName = String.valueOf("package:").concat(UtilityConfig.COMPONENT_PKG);
			String added = Intent.ACTION_PACKAGE_ADDED;
			String removed = Intent.ACTION_PACKAGE_REMOVED;
			String replaced = Intent.ACTION_PACKAGE_REPLACED;
			if (added.equals(action) || removed.equals(action) || replaced.equals(action)) 
			{
				if (plusPackageName.equals(packageName) && SpeechUtility.getUtility() != null)
				{
					SpeechUtility.getUtility().checkServiceInstalled();
				}
			}
		}
	}
	
	/**
	 * <h4>检查服务安装状态</h4>
	 * <script>gShowServiceCheck();</script>
	 * 
	 * @return true, 已安装；false，未安装。
	 * 
	 * @see SpeechConstant#ENGINE_MODE
	 */
	public boolean checkServiceInstalled(){
		boolean isInstalled = false;
		int versionCode = -1;
		try {
			PackageInfo packageInfo = mContext.getPackageManager().getPackageInfo(UtilityConfig.COMPONENT_PKG, 0);
			if(null != packageInfo)
			{
				isInstalled = true;
				versionCode = packageInfo.versionCode;
			}
		} catch (NameNotFoundException e) {
		} catch (Throwable e) {
		}
		
		//判断语记安装状态和版本号变化
		if(isInstalled != serviceInstalled || mVersionCode != versionCode)
		{
			serviceInstalled = isInstalled;
			mVersionCode = versionCode;
			if(null != SpeechRecognizer.getRecognizer())
			{
				SpeechRecognizer.getRecognizer().reInit(mContext);
			}

			if(null != SpeechSynthesizer.getSynthesizer())
			{
				SpeechSynthesizer.getSynthesizer().reInit(mContext);
			}
			
			if(null != SpeechUnderstander.getUnderstander())
			{
				SpeechUnderstander.getUnderstander().reInit(mContext);
			}
			
			if(null != TextUnderstander.getTextUnderstander())
			{
				TextUnderstander.getTextUnderstander().reInit(mContext);
			}
		}
		
		return isInstalled;
	}

	/**
	 * 取得组件信息。
	 * @param action
	 */
	private void getComponentInfo(String action) {
		if (TextUtils.isEmpty(action)) {
			return;
		}
		PackageManager pm = mContext.getPackageManager();
		Intent intent = new Intent(action);
		intent.setPackage(UtilityConfig.COMPONENT_PKG);
		// 组件
		List<ResolveInfo> infos = pm.queryIntentServices(intent, PackageManager.GET_META_DATA |
				                                                 PackageManager.GET_INTENT_FILTERS | 
				                                                 PackageManager.GET_RESOLVED_FILTER);
		if (infos != null && infos.size() > 0) 
		{
			for (ResolveInfo info : infos) 
			{
				String pkgname = info.serviceInfo.packageName;
				SpeechComponent component = putSpeechComponent(pkgname);
				if (component != null) {
					// 这里有可能出现用户写错的可能，导致拿不到正确的数据
					try {
						// 取得引擎信息
						String[] strEngines = info.serviceInfo.metaData
								.getString(UtilityConfig.METADATA_KEY_ENGINE_TYPE).split(",");
						for (String engine : strEngines) {
							component.addEngine(engine);
						}
					} catch (Exception e) {
						DebugLog.LogE(e);
					}
				}
			}
		}
	}

	/**
	 * 把组件加入组件信息列表。
	 * @param packageName
	 * @return 组件
	 */
	private SpeechComponent putSpeechComponent(String packageName) {
		if (TextUtils.isEmpty(packageName)) {
			return null;
		}
		boolean hasFlag = false;
		SpeechComponent componentRet = null;
		// 判断组件列表信息中是否已经有此组件
		for (SpeechComponent component : mAllComponents) {
			if (packageName.equals(component.getPackageName())) {
				hasFlag = true;
				break;
			}
		}
		// 组件信息列表中还没有此组件时，加入
		if (!hasFlag) {
			componentRet = new SpeechComponent(packageName);
			mAllComponents.add(componentRet);
		}
		return componentRet;
	}

	/**
	 * <h4>查询本地组件支持的引擎列表</h4>
	 * <p>
	 * 		引擎列表：local.tts，cloud.tts，local.asr，cloud.asr，local.nlu，cloud.nlu<br>
	 * 更多请参考开发者网站：<script>gShowDeveloperWebsite();</script>。
	 * 如果返回为空，则表示需要提示用户下载组件，如果返回的引擎不满足需求，则通知组件下载引擎。
	 * </p>
	 * 
	 * @return 本地支持的引擎列表
	 */
	public String[] queryAvailableEngines() {
		// 需要重新初始化，防止下载后检测不到的情况
		mAllComponents.clear();
		initComponentInfo();
		
		ArrayList<String> strComponentList = new ArrayList<String>();
		
		for (SpeechComponent component : mAllComponents) {
		    strComponentList.addAll(component.getEngines()); 
		}
		String[] results = new String[strComponentList.size()];
		for (int i = 0; i < strComponentList.size() ; i ++){
		    results[i] = strComponentList.get(i);
		}
		return results;
	}

	/**
	 * <h4>打开引擎设置下载页面</h4>
	 * <script>gShowOpenEngineSettings();</script>
	 * 
	 * @param engineName  引擎名称
	 * @return 错误码，0表示成功
	 */
	public int openEngineSettings(String engineName) 
	{
		try {
			Intent intent = new Intent();
			intent.setPackage(UtilityConfig.COMPONENT_PKG);
			
			// 默认打开老版本语记主界面（包括厂商版本）
			String action = UtilityConfig.COMPONENT_PKG;
			
			if(SpeechConstant.ENG_TTS.equals(engineName)
				&& isPlusActionSupport(UtilityConfig.SETTINGS_ACTION_TTS))
				action = UtilityConfig.SETTINGS_ACTION_TTS;
			else if(SpeechConstant.ENG_ASR.equals(engineName)
					&& isPlusActionSupport(UtilityConfig.SETTINGS_ACTION_ASR))
				action = UtilityConfig.SETTINGS_ACTION_ASR;
			else if(isPlusActionSupport(UtilityConfig.SETTINGS_ACTION_MAIN))
				action = UtilityConfig.SETTINGS_ACTION_MAIN;
			
			intent.setAction(action);
			intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);  
			mContext.startActivity(intent);
		} catch (Exception e) {
			DebugLog.LogE(e);
			return ErrorCode.ERROR_ENGINE_NOT_SUPPORTED;
		}
		
		return ErrorCode.SUCCESS;
	}
	
	/*
	 * 检测语记是否包含支持action的Activity
	 *
	 */
	private boolean isPlusActionSupport(String action)
	{
	   final PackageManager packageManager = mContext.getPackageManager();
	   Intent intent = new Intent(action);
	   intent.setPackage(UtilityConfig.COMPONENT_PKG);
	   List<ResolveInfo> list = packageManager.queryIntentActivities(intent,
	            PackageManager.GET_ACTIVITIES);
	   return list.size() > 0;
	}

	/**
	 * <h4>直接获取组件下载地址</h4>
	 */
	public String getComponentUrl() {
		// modified by yjzhao @ 2014.4.17 升级协议内容 
		final StringBuffer url = new StringBuffer(UtilityConfig.COMPONENT_URL);
		String param = UtilityConfig.getComponentUrlParam(mContext);
		String b64 = Base64.encodeToString(param.getBytes(), Base64.DEFAULT);
		b64 = URLEncoder.encode(b64);
		UtilityConfig.appendHttpParam(url, "key",b64);
		UtilityConfig.appendHttpParam(url, "version","2.0");
		return url.toString();
	}
	
	/**
	 * <h4>设置MSC引擎的状态参数</h4>
	 * 
	 * @see SpeechConstant
	 */
	@Override
	public boolean setParameter(String key, String value) {

		if( TextUtils.isEmpty(key) || TextUtils.isEmpty(value) ){
			return false;
		}
		
		// 首先将本地引擎的参数保存
		super.setParameter(key, value);
		if(!MSC.isLoaded() || SpeechConstant.PARAMS.equals(key))
			return true;
		
		// 设置MSC的信息,需要更改本地引擎参数
		try {
			byte[] keyb  = key.getBytes(DataUtil.UTF8);
			byte[] valb  = value.getBytes(DataUtil.UTF8);
			int ret = MSC.QMSPSetParam(keyb, valb);
			return ret == ErrorCode.SUCCESS;
		} catch (Exception e) {
			DebugLog.LogE(e);
		}catch (UnsatisfiedLinkError e) {
			DebugLog.LogE(e);
		}
		return false;
	}

	/**
	 * <h4>获取MSC引擎的状态参数</h4>
	 * 
	 * @see SpeechConstant
	 */
	@Override
	public String getParameter(String key) {
		if( TextUtils.isEmpty(key) ){
			return null;
		}
		
		// 获取缓存信息，如果存在直接返回
		if(mSessionParams.hasKey(key))
		{
			return super.getParameter(key);
		}
		//@{ start: 去除语记支持
		/*//获取语记的（合成、识别、唤醒、听写、所有）资源信息
		if(key.equals(SpeechConstant.PLUS_LOCAL_TTS)||key.equals(SpeechConstant.PLUS_LOCAL_ASR)
				|| key.equals(SpeechConstant.PLUS_LOCAL_ALL) || key.equals(SpeechConstant.PLUS_LOCAL_IVW))
		{
			try {
				return getPlusLocalInfo(key);
			} catch (Exception e) {
				//避免再次try catch,直接返回字符串
				return "{"+TAG_RESOURCE_RET+":"+ErrorCode.ERROR_INVALID_RESULT+"}";
			}
		}else {*/
			if(!MSC.isLoaded())
			{
				return null;
			}
			// 获取MSC的信息
			try {
				if(key.equals(MscKeys.VER_MSC)||key.equals(MscKeys.VER_ASR)
						|| key.equals(MscKeys.VER_TTS)||key.equals(MscKeys.VER_IVW)) {
					byte[] keyb = key.getBytes(DataUtil.UTF8);
					MSCSessionInfo info = new MSCSessionInfo();
					String ver = new String(MSC.QMSPGetVersion(keyb, info), DataUtil.UTF8);
					if(info.errorcode==ErrorCode.SUCCESS)
						return ver;
				} else {
					byte[] keyb  = key.getBytes(DataUtil.UTF8);
					MSCSessionInfo info = new MSCSessionInfo();
					int ret = MSC.QMSPGetParam(keyb, info);
					if(ret == ErrorCode.SUCCESS)
						return new String(info.buffer,DataUtil.UTF8);
				}
			} catch (Exception e) {
				DebugLog.LogS(e);
			} catch (UnsatisfiedLinkError e) {
				DebugLog.LogS(e);
			}
		//}
		return null;
	}
	
	/**
	 * <h4><script>gShowGetPlusLocalInfoTitle();</script></h4>
	 * @deprecated 此接口不再支持
	 * <p>
	 * 		key可设置的参数有:
	 * <br>{@link SpeechConstant#PLUS_LOCAL_ALL}：所有资源
	 * <br>{@link SpeechConstant#PLUS_LOCAL_TTS}：合成资源}
	 * <br>{@link SpeechConstant#PLUS_LOCAL_ASR}：识别资源}
	 * <br>{@link SpeechConstant#PLUS_LOCAL_IVW}：唤醒资源}
	 * </p>
	 * @param key
	 * @return
	 * @throws JSONException
	 */
	public String getPlusLocalInfo(String key) throws JSONException{
		JSONObject localResData = new JSONObject();
		String content = "";
		//语记安装检测，未安装提示安装语记
		if(!checkServiceInstalled())
		{
			localResData.put(TAG_RESOURCE_RET, ErrorCode.ERROR_COMPONENT_NOT_INSTALLED);
			return localResData.toString();
		}
		//语记版本检测，版本过低提示升级语记版本
		if(getServiceVersion() < GETPARAMETER_SUPPORTED_VER)
		{
			localResData.put(TAG_RESOURCE_RET, ErrorCode.ERROR_VERSION_LOWER);
			return localResData.toString();
		}
		//语记版本检测，厂商版本提示系统预装
		else if(GETPARAMETER_NOT_SUPPORTED_LOWER <= getServiceVersion()&& getServiceVersion() <= GETPARAMETER_NOT_SUPPORTED_UPPER) {
			localResData.put(TAG_RESOURCE_RET, ErrorCode.ERROR_SYSTEM_PREINSTALL);
			return localResData.toString();
		}
		
		ContentResolver mResolver = mContext.getContentResolver();
		Uri uri = Uri.parse(LOCAL_RESOURCE_URI);
		
		Cursor mCursor =mResolver.query(uri, null, key, null, null);
		int index = mCursor.getColumnIndex(TAG_RESOURCE_CONTENT);
		if(mCursor != null && mCursor.moveToFirst())
		{
			//列名不能在getString中获取，因为这样游标会越界，用index固定列名的位置。
			content = mCursor.getString(index);
			Log.v(TAG, content);
		}
		//关闭游标
		if (mCursor != null) {
			mCursor.close();
		}
		
		//获取到空串
		if (TextUtils.isEmpty(content)) {
			localResData.put(TAG_RESOURCE_RET,ErrorCode.ERROR_INVALID_RESULT);
			return localResData.toString();
		}
		//正常返回
		localResData.put(TAG_RESOURCE_RET,ErrorCode.SUCCESS);
		localResData.put(TAG_RESOURCE_RESULT,new JSONObject(content));
		return localResData.toString();
	}
	
	
	/**
	 * <h4>获得引擎选择模式</h4>
	 * 
	 * @return 引擎模式
	 * 
	 * @see SpeechConstant#ENGINE_MODE
	 */
	public ENGINE_MODE getEngineMode()
	{
		return engineMode;
	}
	
	/**
	 * <h4>获取服务的版本信息</h4>
	 * <p>
	 * 		<script>gShowServiceVersion();</script>
	 * </p>
	 * @return 服务版本号
	 */
	public int getServiceVersion() 
	{
		if(mVersionCode < 0) {
			try {
				PackageInfo packageInfo = mContext.getPackageManager().getPackageInfo(UtilityConfig.COMPONENT_PKG, 0);
				if(null != packageInfo) {
					mVersionCode = packageInfo.versionCode;
				}
			} catch (NameNotFoundException e) {
			}
		}
		return mVersionCode;
	}
	
	//判断是否是应用自身的进程，排除更新和百度推送的进程多次login等操作
	private static boolean isOwnProcess(Context mContext)
	{
		try {
			//获取当前进程id
			int pid = android.os.Process.myPid();
			ActivityManager activityManager = (ActivityManager)mContext.getSystemService(Context.ACTIVITY_SERVICE);
			if(activityManager != null)
			{
				for (ActivityManager.RunningAppProcessInfo appProcess : activityManager.getRunningAppProcesses())
				{
					if (appProcess.pid == pid) 
					{
						DebugLog.LogD("process name:" + appProcess.processName);
						ApplicationInfo appinfo = mContext.getPackageManager().getApplicationInfo(mContext.getPackageName(), 0);
						if(appinfo.packageName.equals(appProcess.processName))
						{
							DebugLog.LogD("process name:" + appProcess.processName+"is own process");
							return true;
						}
					}
				}
			}
		} catch (Exception e) {
			DebugLog.LogE(e);
		}
		return false;
	}
}
