package com.iflytek.cloud;

import org.json.JSONObject;

import com.iflytek.cloud.msc.util.log.DebugLog;

/**
 * <h3>声纹结果</h3>
 * <p>
 * 		本类通过构造函数{@link #VerifierResult(String)}，解析返回的结果信息到类的成员变量中，
 * 应用可通过{@link #source}获取返回的结果信息源文本，或通过{@link #ret}等直接获取对应的
 * 参数值。
 * </p>
 * 
 * @since version:1073
 * @author <script>gShowAuthor();</script>
 * @see SpeakerVerifier
 * @see VerifierListener#onResult
 */
public class VerifierResult {
	public static final String TAG = "VerifyResult";
	// 以下是错误码err的取值
	/** 内核异常*/
	public static final int MSS_ERROR_IVP_GENERAL = 11600;
	/** rgn超过最大支持次数*/
	public static final int MSS_ERROR_IVP_EXTRA_RGN_SOPPORT = 11601;	
	/** 音频波形幅度太大，超出系统范围，发生截幅*/
	public static final int MSS_ERROR_IVP_TRUNCATED = 11602;
	/** 太多噪音*/
	public static final int MSS_ERROR_IVP_MUCH_NOISE = 11603;
	/** 声音太小*/
	public static final int MSS_ERROR_IVP_TOO_LOW = 11604;
	/** 没检测到音频*/ 
	public static final int MSS_ERROR_IVP_ZERO_AUDIO = 11605;
	/** 音频太短*/
	public static final int MSS_ERROR_IVP_UTTER_TOO_SHORT = 11606;
	/** 音频内容与给定文本不一致*/
	public static final int MSS_ERROR_IVP_TEXT_NOT_MATCH = 11607;
	/** 音频长达不到自由说的要求*/
	public static final int MSS_ERROR_IVP_NO_ENOUGH_AUDIO =11608;

	/**
	 * <h4>业务类型</h4>
	 * <p>
	 * 		业务类型，包括注册("train")，验证("verify")。与业务请求时一致。
	 * </p>
	 * 
	 * <p>是否一定出现：是</p>
	 */
	public String   sst;
	
	/**
	 * <h4>返回值</h4>
	 * <p>
	 * 		类似于错误码，与{@link VerifierListener#onError(SpeechError)}中的
	 * 错误信息包含的错误码类似。返回值为成功
	 * （{@link ErrorCode#SUCCESS}）时，如果是验证业务，则表示验证成功，
	 * 如果是注册还要参考{@link #err}的值，如果{@link #err}存在，且不为0，则
	 * 表示有错误，需要根据错误码进行相应的处理。
	 * </p>
	 * <p>是否一定出现：是</p>
	 * @see #err
	 */
	public int 	ret = 0;
	
	/**
	 * <h4>描述信息</h4>
	 * <p>
	 * 		描述信息为描述当前返回结果的可选字段，如不需要，可直接忽略。
	 * </p>
	 * <p>是否一定出现：否</p>
	 */
	public String   dcs = "";
	
	/**
	 * <h4>得分</h4>
	 * <p>
	 * 		得分为验证通过的参考，得分越高，验证的一致性越吻合。应用层根据
	 * {@link #ret}值即可判断是否通过，得分值仅用于参考。
	 * </p>
	 * <p>是否一定出现：在验证时是，注册时不出现。</p>
	 */
	public Double score ;
	
	/**
	 * <h4>原始得分</h4>
	 * <p>
	 * 		应用一般用不到此参数，可直接忽略。
	 * </p>
	 * <p>是否一定出现：在验证时是，注册时不出现。</p>
	 */
	public Double score_raw;
	
	// 注册信息
	/**
	 * <h4>声纹ID</h4>
	 * <p>
	 * 		声纹ID在注册成功时由引擎生成，并返回给应用。声纹ID在应用使用
	 * 声纹时暂时不需要传入。
	 * </p>
	 * <p>是否一定出现：注册成功和验证时，一定出现</p>
	 */
	public String   vid = "";
	
	/**
	 * <h4>注册成功次数</h4>
	 *  <p>
	 *  		当前注册的成功次数，当注册成功次数与{@link #rgn 需要注册次数}
	 *  一致时，注册完成。
	 *  </p>
	 * <p>是否一定出现：注册时一定出现，验证时不出现</p>
	 * @see #rgn
	 */
	public int suc = 0;
	
	/**
	 * <h4>需要注册次数</h4>
	 * <p>
	 * 		除了自由说方式外，随机密码和固定文本方式，均需要多次注册，以获取用户的声纹
	 * 特征。关于使用什么方式，请参考密码类型参数{@link SpeechConstant#ISV_PWDT}，
	 * 关于注册次数设置，请参考注册次数参数{@link SpeechConstant#ISV_RGN}。
	 * </p>
	 * <p>是否一定出现：注册时一定出现，验证时不出现</p>
	 * @see #suc
	 */
	public int rgn = 0;
	
	/**
	 * <h4>注册完成描述信息</h4>
	 * <p>
	 * 		当注册完成时，返回此字段，值为"ok"，用于描述注册完成。实际完成注册，可通过
	 * 对比{@link #suc}与{@link #rgn}相等即可。
	 * </p>
	 * <p>是否一定出现：否</p>
	 */
	public String trs = "";
	
	/**
	 * </h4>注册返回的错误码</h4>
	 * <p>
	 * 		在注册时，返回值{@link #ret}为{@link ErrorCode#SUCCESS}时，还要参考此值，
	 * 查看当前注册过程是否成功，若此值与{@link ErrorCode#SUCCESS}不一致，则应根
	 * 据{@link #suc}获取已完成注册的次数，根据错误码，修正并重试应该注册的那次（即
	 * 重试第suc+1那次）。
	 * </p>
	 * <p>是否一定出现：否</p>
	 */
	public int err = 0;
	
	/**
	 * </h4>服务端返回的原始结果文本</h4>
	 * <p>
	 * 		原始文本用于调试时查看返回结果文本是否正确，应用层可忽略此成员。
	 * </p>
	 * <p>是否一定出现：是</p>
	 */
	public String source = "";
	
	public VerifierResult(String text)
	{
		try
		{	
			source = text;
			JSONObject joResult 	= new JSONObject(source);
			DebugLog.LogD("VerifyResult = " + source);
			if(joResult.has("ret"))
				ret = joResult.getInt("ret");
			if(joResult.has("sst"))
				sst = joResult.getString("sst");
			if(joResult.has("dcs"))
				dcs = joResult.getString("dcs");
			if(joResult.has("suc"))
				suc = joResult.getInt("suc");
			if(joResult.has("vid"))
				vid = joResult.getString("vid");
			if(joResult.has("rgn"))
				rgn = joResult.getInt("rgn");
			if(joResult.has("trs"))
				trs = joResult.getString("trs");
			if(joResult.has("err"))
				err = joResult.getInt("err");
			if (joResult.has("score")) {
				score = joResult.getDouble("score");
			}
			if (joResult.has("score_raw")) {
				score_raw = joResult.getDouble("score_raw");
			}
		}
		catch(Exception e)
		{
			DebugLog.LogE(e);
		}
	}
}
