package com.iflytek.cloud.msc.ise;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.Locale;
import java.util.concurrent.ConcurrentLinkedQueue;

import android.content.Context;
import android.media.MediaRecorder.AudioSource;
import android.os.Bundle;
import android.os.HandlerThread;
import android.os.Message;
import android.os.SystemClock;
import android.text.TextUtils;

import com.iflytek.cloud.ErrorCode;
import com.iflytek.cloud.EvaluatorListener;
import com.iflytek.cloud.EvaluatorResult;
import com.iflytek.cloud.SpeechConstant;
import com.iflytek.cloud.SpeechError;
import com.iflytek.cloud.SpeechEvent;
import com.iflytek.cloud.msc.module.MscHandler;
import com.iflytek.cloud.msc.module.SessPerflog;
import com.iflytek.cloud.msc.module.MscSession.ResultStatus;
import com.iflytek.cloud.msc.util.DataUtil;
import com.iflytek.cloud.msc.util.NetworkUtil;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.msc.util.log.PerfLogger;
import com.iflytek.cloud.param.HashParam;
import com.iflytek.cloud.param.MscKeys;
import com.iflytek.cloud.record.FilePcmRecorder;
import com.iflytek.cloud.record.PcmRecorder;
import com.iflytek.cloud.record.PcmRecorder.PcmRecordListener;

/**
 * Evaluator thread, manager record and msc interface
 * 
 * @author iflytek
 * 
 */
public class MscEvaluator extends MscHandler implements PcmRecordListener {
	
	// Upstream and downstream traffic generated duration the session
	public static int mUpflow =  0;
	
	public static int mDownflow = 0;

	private static Boolean isAnswerTextByte = false;
	
	private volatile EvaluatorListener mEvaluatorListener = null;

	long mRelatRecordStart  = 0;

	// 录音源设置为麦克风
	protected int mAudioSource = AudioSource.MIC;
	
	protected IseSession mEvaSession = new IseSession();
	
	protected PcmRecorder mPcmRecorder = null;
	
	// 性能结构化日志
	protected SessPerflog mPerflog = new SessPerflog();
	
	protected String mTextParams = null;
	
	protected byte[] mAnswerTextBytes = null;

	protected String mAnswerText = null;
	
	protected String mUserModelId = null;
	
	private ConcurrentLinkedQueue<byte[]> mRecordBuffer = null;
	
	private ConcurrentLinkedQueue<byte[]> mRecordQueue = null;
	
	private ArrayList<String> mResults = null;
	
	// 用户是否主动点击了停止录音
	private boolean mUstopRecord = false;
	
	private ResultStatus mResultStatus = ResultStatus.noResult;
	private final static int NO_RESULT_DELAY = 20;
	
	private String mSessionID = null;
	
	private boolean mUseBluetooth = false;
	
	public MscEvaluator(Context context, HashParam param, HandlerThread handlerthread) {
		super(context, handlerthread);
		mRecordQueue = new ConcurrentLinkedQueue<byte[]>();
		mRecordBuffer = new ConcurrentLinkedQueue<byte[]>();
		mResults = new ArrayList<String>();
		mUstopRecord = false;

		setParams(param);
	}
	
	public MscEvaluator(Context context, ConcurrentLinkedQueue<byte[]> buffer,
			HashParam param, HandlerThread handlerthread) {
		super(context, handlerthread);
		mRecordBuffer = new ConcurrentLinkedQueue<byte[]>();
		mRecordQueue = buffer;
		mResults = new ArrayList<String>();
		mUstopRecord = false;

		setParams(param);
	}
	
	public MscEvaluator(Context context, byte[] buffer, HashParam param,
			HandlerThread handlerthread) {
		super(context, handlerthread);

		mRecordBuffer = new ConcurrentLinkedQueue<byte[]>();
		mRecordQueue = new ConcurrentLinkedQueue<byte[]>();
		mResults = new ArrayList<String>();

		ArrayList<byte[]> array = DataUtil.splitBuffer(buffer, 0, buffer.length, 5120);
		for (int i = 0; i < array.size(); i++) {
			mRecordQueue.add(array.get(i));
		}

		setParams(param);
	}

	public synchronized void startListening(String answer, String textParams, EvaluatorListener listener) {
		isAnswerTextByte = false;
		mAnswerText = answer;
		mTextParams = textParams;
		mUserModelId = getParam().getString(SpeechConstant.ISE_USER_MODEL_ID);
		mEvaluatorListener = listener;

		DebugLog.LogD("[ise]startListening called 01");
		// start session
		start();
	}

	
	public synchronized void startListening(byte[] answer, String textParams, EvaluatorListener listener) {
		isAnswerTextByte = true;
		mAnswerTextBytes = answer;
		mTextParams = textParams;
		mUserModelId = getParam().getString(SpeechConstant.ISE_USER_MODEL_ID);
		mEvaluatorListener = listener;

		DebugLog.LogD("[ise]startListening called 02");
		// start session
		start();
	}

	public synchronized boolean stopListening(boolean userStop) {
		if (getStatus() != Status.recording) {
			DebugLog.LogD("stopRecognize fail  status is :" + getStatus());
			return false;
		}
		
		if (mPcmRecorder != null) {
			mPcmRecorder.stopRecord(getParam().getBoolean(MscKeys.RECORD_FORCE_STOP, 
					false));
		}
		//user stop the record
		mUstopRecord = userStop;
		sendMsg(MSG_RECORD_STOPD);
		
		return true;
	}

	@Override
	public void cancel(boolean notifyError) {
		if (notifyError && isRunning() && mEvaluatorListener != null) {
			mEvaluatorListener.onError(new SpeechError(ErrorCode.ERROR_INTERRUPT));
		}
		releaseRecord();

		super.cancel(notifyError);
	}
	
	@Override
	protected void onMsgProcess(Message msg) throws Throwable, SpeechError {
		super.onMsgProcess(msg);

		switch (msg.what) {
		case MSG_START:
			proc_Msg_Start();
			break;
		case MSG_SESSION_BEGIN:
			proc_Msg_Session_Begin();
			break;
		case MSG_RECORD_AUDIO:
			proc_Msg_Record_Data(msg);
			break;
		case MSG_RECORD_STOPD:
			proc_Msg_Record_Stoped();
			break;
		case MSG_RESULT:
			proc_Msg_Result(msg);
			break;
		case MSG_SPEECH_TIMEOUT:
			DebugLog.LogD("--->on timeout vad");
			vadEndCall();
		case MSG_NET_PERF:
			// proc_Msg_Netperf();
			break;
		default:
			break;
		}
	}
	
	/**
	 * 启动消息处理
	 * @throws Exception
	 */
	protected void proc_Msg_Start() throws Exception {	
		DebugLog.LogD("--->onStart: in");
		
		boolean net_check = getParam().getBoolean(SpeechConstant.NET_CHECK, true);
		if(net_check) {
			NetworkUtil.checkNetwork(mContext);
		}
		
		int read_rate = getParam().getInt(MscKeys.KEY_RECORD_READ_RATE,
				PcmRecorder.READ_INTERVAL40MS);

		mAudioSource = getParam().getInt(SpeechConstant.AUDIO_SOURCE, AudioSource.MIC);
		// 不是外部写录音会话,开启录音机
		if (mAudioSource != MscKeys.VALUE_AUDIO_EXTERNAL && isRunning()) {
			DebugLog.LogD("[ise]start  record");
			
			if (mAudioSource == MscKeys.VALUE_AUDIO_EXTERNALFILE) {
				//外部指定音频文件
				String path_source = getParam().getString(SpeechConstant.ISE_SOURCE_PATH);
				mPcmRecorder = new FilePcmRecorder(getSampleRate(), read_rate, mAudioSource, path_source);
			} else {
				mUseBluetooth = getParam().getBoolean( SpeechConstant.BLUETOOTH, mUseBluetooth );
				if( mUseBluetooth ){
					startBluetooth();
				}
				mPcmRecorder = new PcmRecorder(getSampleRate(), read_rate, mAudioSource);
			}
			
			mPcmRecorder.startRecording(MscEvaluator.this);
		}

		if (getStatus() != Status.exiting && null != mEvaluatorListener)
			mEvaluatorListener.onBeginOfSpeech();
		
		// 语音输入超时消息
		removeMessages(MSG_SPEECH_TIMEOUT);
		if (-1 != mSpeechTimeOut) {
			sendMsg(MSG_SPEECH_TIMEOUT, Priority.normal, false, mSpeechTimeOut);
		}
		
		// 插入到消息头部，防止音频被预先处理
		sendMsg(MSG_SESSION_BEGIN, Priority.max, false, 0);
		
		DebugLog.LogD("--->onStart: out");
	}
	
	/**
	 * MSC SessionBegin消息处理
	 * 特别注意点：缓冲音频的写入，必须保证在SessionBegin之后
	 * @throws Exception
	 */
	protected void proc_Msg_Session_Begin() throws Exception {
		// 注册时不需要多次启动会话，后续需要考虑sessionBegin失败情况！！
		if(mEvaSession.mClientID == null){
			PerfLogger.appendInfo(PerfLogger.SDK_SESSION_BIGNE, null);
			mEvaSession.sessionBegin(mContext, mUserModelId, this);
		}
		
		byte[] data = null;
		if (isAnswerTextByte) {
			if ("1".equals(getParam().getString(SpeechConstant.TEXT_BOM))) {
				// 给试题加上bom头
				data = DataUtil.getUTF8Bom(mAnswerTextBytes);
			} else {
				data = mAnswerTextBytes;
			}
		} else {
			if ("1".equals(getParam().getString(SpeechConstant.TEXT_BOM))) {
				data = DataUtil.getUTF8Bom(mAnswerText);
			} else {
				data = mAnswerText.getBytes(DataUtil.GB2312);
			}
		}
		mEvaSession.putText(data, TextUtils.isEmpty(mTextParams)? null: mTextParams.getBytes(DataUtil.GB2312));
		setStatus(Status.recording);
		
		sendMsg(MSG_RESULT, Priority.normal, false, NO_RESULT_DELAY);
	}
	
	/**
	 * 录音数据通知消息
	 * @param msg
	 * @throws Exception
	 */
	protected void proc_Msg_Record_Data(Message msg) throws Exception{
		DebugLog.LogD("proc_Msg_Record_Data");
		byte[] data = (byte[])msg.obj;
		if (data == null || data.length == 0)
			return;
		
		if (!TextUtils.isEmpty(getParam().getString(SpeechConstant.ISE_AUDIO_PATH))) {
			// 保存录音数据，用以再次识别和保存
			mRecordBuffer.add(data);
		}

		uploadData(data, true);
	}
	
	/**
	 * 录音停止的通知消息
	 * @throws SpeechError
	 * @throws IOException
	 * @throws InterruptedException
	 */
	private void proc_Msg_Record_Stoped() throws SpeechError, IOException, InterruptedException {
		DebugLog.LogD("--->onStoped: in");
		if (!isRunning())
			releaseRecord();

		// upload end flag
		mEvaSession.pushEndFlag();

		// 最低优先级消息，会被自动清除
		updateTimeoutMsg();

		DebugLog.LogD("--->onStoped: out");
	}
	
	/**
	 * 结果通知消息
	 * 	status:   2----unknown 0----has result 5----is last result
	 * @throws SpeechError
	 * @throws InterruptedException
	 * @throws UnsupportedEncodingException
	 */
	void proc_Msg_Result(Message msg) throws SpeechError, InterruptedException, UnsupportedEncodingException{
		requestResult();
		
		if( ResultStatus.noResult == mResultStatus ){
			sendMsg(MSG_RESULT, Priority.normal, false, NO_RESULT_DELAY);
		}else if( ResultStatus.hasResult == mResultStatus ){
			sendMsg( MSG_RESULT );
		}
	}
	
	@Override
	protected void onEnd(SpeechError error) {
		DebugLog.LogD("--->onEnd: in");
		releaseRecord();
		
		getSessionID();
		
		PerfLogger.appendInfo(PerfLogger.SESSION_END_BEGIN, null);
		
		if(mUserCancel)
			mEvaSession.sessionEnd("user abort");
		else if(error != null)
			mEvaSession.sessionEnd("error" + error.getErrorCode());
		else
			mEvaSession.sessionEnd("success");
		PerfLogger.appendInfo(PerfLogger.SESSION_END_END, null);

		super.onEnd(error);

		if (mEvaluatorListener != null) {
			if (mUserCancel) // cancel by user
			{
				// DebugLog.LogD("VerifyListener#onCancel");
				// mListener.onCancel();
			} else // auto exit
			{
				DebugLog.LogD("VerifyListener#onEnd");
				
				if (error != null) {
					// 通过onEvent抛出服务端会话id
					Bundle bundle = new Bundle();
					bundle.putString(SpeechEvent.KEY_EVENT_SESSION_ID, getSessionID());
					mEvaluatorListener.onEvent(SpeechEvent.EVENT_SESSION_ID, 0, 0, bundle);
					
					mEvaluatorListener.onError(error);
				}
			}
		}
		mEvaluatorListener = null;
		DebugLog.LogD("--->onEnd: out");
	}

	private void requestResult() throws SpeechError, UnsupportedEncodingException {
		ResultStatus status = mEvaSession.getResult();
		mResultStatus = status;
		switch (status) {
		case noResult:
			break;
		case hasResult:
			notifyResult(false);
			break;
		case resultOver:
			notifyResult(true);
			break;
		default:
			break;
		}
	}

	/**
	 * called from VAD check
	 */
	private void vadEndCall() {
		if (Status.recording == getStatus()) {
			DebugLog.LogD( "Ise Msc vadEndCall" );
			stopListening(false);
			if (mEvaluatorListener != null) {
				mEvaluatorListener.onEndOfSpeech();
			}
		}
	}

	/**
	 * Notify user about ISE result.
	 * 
	 * @param isLast
	 * @throws SpeechError
	 * @throws UnsupportedEncodingException
	 */
	private void notifyResult(boolean isLast) throws SpeechError, UnsupportedEncodingException{
		
		mStatusBegin = SystemClock.elapsedRealtime();
		// 保存识别结果,用以判断是否返回10118错误
		if(mEvaSession.getResultData() != null && mEvaSession.getResultData().length > 0) {
			final String result = new String(mEvaSession.getResultData(), DataUtil.UTF8);
			mResults.add( result );

		}
		
		//TODO : not clear
		notifyEngineResult(isLast);
	}
	
	
	//TODO : not clear
	public void notifyEngineResult(boolean isLast) throws SpeechError, UnsupportedEncodingException{
		DebugLog.LogD("msc result time:" + System.currentTimeMillis());
		//这里设置一个默认值为GB2312，后期服务器如果改动可以在外部进行设置
		String encoding = getParam().getString("rse", DataUtil.GB2312);
		String resString = new String(mEvaSession.getResultData(), encoding);
		EvaluatorResult res = new EvaluatorResult(resString);
		
		if (null != mEvaluatorListener) {
			// 通过onEvent抛出服务端会话id
			Bundle bundle = new Bundle();
			bundle.putString(SpeechEvent.KEY_EVENT_SESSION_ID, getSessionID());
			mEvaluatorListener.onEvent(SpeechEvent.EVENT_SESSION_ID, 0, 0, bundle);
			

			PerfLogger.appendInfo(PerfLogger.GET_RESULT, null);
			mEvaluatorListener.onResult(res, isLast);
		}
		
		// 会话结束
		if (isLast) {
			exit(null);
		}
	}
	
	@Override
	protected void onParseParam() {
		mSpeechTimeOut = getParam().getInt(SpeechConstant.KEY_SPEECH_TIMEOUT, -1);
		DebugLog.LogD("mSpeechTimeOut=" + mSpeechTimeOut);
		
		if (DataUtil.UTF8.equals(getParam().getString(SpeechConstant.TEXT_ENCODING))
				&& Locale.CHINA.toString().equalsIgnoreCase(getParam().getString(SpeechConstant.LANGUAGE))) {
			// 当前进中文评测时，若没有设置text_bom值，则需要加上bom头
			getParam().putParam(SpeechConstant.TEXT_BOM, "1", false);
		} else {
			// 当外没有设置text_bom时，设为0
			getParam().putParam(SpeechConstant.TEXT_BOM, "0", false);
		}
		super.onParseParam();
	}



	private void releaseRecord() {
		if (mPcmRecorder != null){
			mPcmRecorder.stopRecord(getParam().getBoolean(MscKeys.RECORD_FORCE_STOP, false));
			mPcmRecorder = null;
			
			if( mUseBluetooth ){
				stopBluetooth();
			}
		}
	}

	private void uploadData(byte[] data, boolean needvadNotify) 
		throws SpeechError {	
		/*
		 * will put mSession.mWriteout send to MSC service, the service will
		 * write returning data to mWriteOut.
		 */
		mEvaSession.pushAudioData(data, data.length);

		//check vad state
		if (needvadNotify) {
			int ep = mEvaSession.getEpStatus();
			if (ep == 3) {
				// VAD后端点消息
				vadEndCall();
			} else {
				int volume = mEvaSession.getAudioVolume();
				callbackVolume(data, volume);
			}
		}
	}
	/**
	 * 录音缓存队列
	 * @return
	 */
	public ConcurrentLinkedQueue<byte[]> getBuffer() {
		byte[] data = null;
		while ((data = mRecordQueue.poll()) != null) {
			mRecordBuffer.add(data);
		}
		return mRecordBuffer;
	}

	
	/**
	 * Notify the listener that the volume is change
	 * 
	 * @param data
	 * @param volume
	 */
	private void callbackVolume(byte[] data, int volume) {
		if (mEvaluatorListener != null && isRunning()) {
			mEvaluatorListener.onVolumeChanged(volume,data);
		}
	}
	
	/**
	 * Implements PcmRecordListener. 
	 * 
	 * <p>The PcmRecord will use callback this method to supply audio data. 
	 * After get the audio data, we store the data to mRecordQueue. 
	 * 
	 * <p>The thread that start in MscLooper will read audio data from mRecordQueue, 
	 * if successfully get the audio data, then will use mEvaSession to send the data
	 * to MSC service.
	 * 
	 * @throws IOException
	 */
	@Override
	public void onRecordBuffer(byte[] data, int offset, int length) {
		if (Status.recording != getStatus()) {
			DebugLog.LogE("onRecordBuffer statuts not recording");
			return;
		}

		if (length > 0) {
			// 拷贝内存，发送录音消息
			byte[] temp = new byte[length];
			System.arraycopy(data, offset, temp, 0, length);
			sendMsg(obtainMessage(MSG_RECORD_AUDIO, temp));
		}
	}

	/**
	 * Implements of PcmRecordListener
	 */
	@Override
	public void onError(SpeechError e) {
		exit(e);
	}

	@Override
	public void onRecordStarted(boolean success) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public String getClientID() {
		return mEvaSession.getClientID();
	}
	
	@Override
	public void onRecordReleased() {
		if(mPcmRecorder != null && mPcmRecorder instanceof FilePcmRecorder)
		{
			//文件录音释放录音机则有可能是文件读完，模拟手动停止录音
			stopListening(true);
		}
	}

	@Override
	public String getSessionID() {
		if( TextUtils.isEmpty(mSessionID) ){
			mSessionID = mEvaSession.getSessionID();
		}
		return mSessionID;
	}


	@Override
	public String getTextEncoding() {
		return getParam().getString(SpeechConstant.TEXT_ENCODING, DataUtil.GB2312);
	}
}