package com.iflytek.cloud.msc.isr;

import java.io.IOException;

import com.iflytek.cloud.ErrorCode;
import com.iflytek.cloud.SpeechConstant;
import com.iflytek.cloud.SpeechError;
import com.iflytek.cloud.SpeechEvent;
import com.iflytek.cloud.SpeechUtility;
import com.iflytek.cloud.msc.module.SessPerflog;
import com.iflytek.cloud.msc.util.NetworkUtil;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.param.HashParam;
import com.iflytek.cloud.param.MscKeys;
import com.iflytek.cloud.param.ParamBuilder;
import com.iflytek.cloud.record.FilePcmRecorder;
import com.iflytek.cloud.record.PcmRecorder;
import com.iflytek.cloud.util.AudioDetector;
import com.iflytek.cloud.util.AudioDetector.DetectorResult;

import android.content.Context;
import android.os.HandlerThread;
import android.os.Message;
import android.text.TextUtils;

public class MscRecognizerMeta extends MscRecognizer {
	
	private int mSampleRate = PcmRecorder.RATE16K;
	private AudioDetector mDetector = null;
	boolean mIsStartFound = false;
	
	private static final int BUF_TIME = 300;
	private byte[] mLastBuf = null;
	
	public MscRecognizerMeta(Context context, HashParam param, HandlerThread mHandlerThread) {
		super(context, param, mHandlerThread);
		
		param = this.getParam();
		mSampleRate = null!=param ? param.getInt( SpeechConstant.SAMPLE_RATE, PcmRecorder.RATE16K ) : PcmRecorder.RATE16K;
		
		param.putParam( SpeechConstant.VAD_ENABLE, "0", true );
		
		mDetector = AudioDetector.getDetector();
		if( null == mDetector ){
			final StringBuffer paramBuffer = new StringBuffer();
			paramBuffer.append( SpeechConstant.SAMPLE_RATE ).append( MscKeys.KEY_VAL_SEP ).append( mSampleRate );
			
			final String vadEngine = param.getString( AudioDetector.VAD_ENGINE, AudioDetector.TYPE_META );
			paramBuffer.append( MscKeys.KEY_SEP ).append( AudioDetector.VAD_ENGINE ).append( MscKeys.KEY_VAL_SEP ).append( vadEngine );
			final String resPath = null!=param ? param.getString( AudioDetector.RES_PATH ) : null;
			if( !TextUtils.isEmpty(resPath) ){
				paramBuffer.append( MscKeys.KEY_SEP ).append( AudioDetector.RES_PATH ).append( MscKeys.KEY_VAL_SEP ).append( resPath );
			}
			
			mDetector = AudioDetector.createDetector( context, paramBuffer.toString() );
		}
		
		final int detectorBos = 0<this.mSpeechTimeOut ? mSpeechTimeOut : Integer.MAX_VALUE;
		mDetector.setParameter( SpeechConstant.VAD_BOS, Integer.toString(detectorBos) );
		final String vadEos = param.getString( SpeechConstant.VAD_EOS, Integer.toString(ParamBuilder.getDefaultEos(this)) );
		DebugLog.LogD( "meta vad eos on recog: "+vadEos );
		mDetector.setParameter( SpeechConstant.VAD_EOS, vadEos );
		mDetector.setParameter( SpeechConstant.KEY_SPEECH_TIMEOUT, Integer.toString(mSpeechTimeOut) );
	}
	
	@Override
	protected void proc_Msg_Start() throws Exception {
		DebugLog.LogD("start connecting");
		String engine = getParam().getString(SpeechConstant.ENGINE_TYPE);
		
		//网络检查
		boolean net_check = getParam().getBoolean(SpeechConstant.NET_CHECK, true);
		if(net_check){
			if(SpeechConstant.TYPE_CLOUD.equals(engine))
			{
				NetworkUtil.checkNetwork(mContext);
			} else if(SpeechConstant.TYPE_MIX.equals(engine) || "mixed".equals(engine)){
				//通过try catch判断网络是否连接
				try {
					NetworkUtil.checkNetwork(mContext);
				} catch (Exception e) {
					getParam().putParam(SpeechConstant.ENGINE_TYPE, SpeechConstant.TYPE_LOCAL);
				}
			}
		}
				
		int read_rate = getParam().getInt(MscKeys.KEY_RECORD_READ_RATE, PcmRecorder.READ_INTERVAL40MS);

		setStatus(Status.recording);
		
		final int bufLen = BUF_TIME  * mSampleRate * 2 / 1000;
		DebugLog.LogD( "MscRecognizerMeta last buffer len: "+bufLen );
		mLastBuf = new byte[bufLen];
		
		// 判断不是外部写录音会话
		if(mAudioSource != MscKeys.VALUE_AUDIO_EXTERNAL && isRunning()){
			DebugLog.LogD( "start  record");
			
			if(mAudioSource == MscKeys.VALUE_AUDIO_EXTERNALFILE)
			{
				//外部指定音频文件
				String path_source = getParam().getString(SpeechConstant.ASR_SOURCE_PATH);
				mRecorder = new FilePcmRecorder(getSampleRate(), read_rate, mAudioSource, path_source);
			}
			else
			{
				mRecorder = new PcmRecorder(getSampleRate(), read_rate,mAudioSource);
				// 内部开启录音机时防止外部提前调用stopListening
				if(hasMessages(MSG_RECORD_STOPD))
				{
					throw new SpeechError(ErrorCode.MSP_ERROR_NO_DATA);
				}
			}
			
			mPerflog.pushSessionInfo(SessPerflog.KEY_REC_OPEN);
			mRecorder.startRecording(this);
			
			mSpeechTimeOut = this.getParam().getInt( SpeechConstant.KEY_SPEECH_TIMEOUT, INFINITE ); 
			if( INFINITE != mSpeechTimeOut ){
				// 语音输入超时消息
				sendMsg(MSG_SPEECH_TIMEOUT, Priority.normal, false,mSpeechTimeOut);
			}
		}

		if (null != mListener && mAudioSource > MscKeys.VALUE_AUDIO_EXTERNAL){
			//内部开启录音机时才需要回调
			mListener.onBeginOfSpeech();
		}
		
		if( null == mDetector ){
			throw new SpeechError( ErrorCode.ERROR_ENGINE_INIT_FAIL );
		}else{
			mDetector.reset();
		}
	}//end of function proc_Msg_Start
	
	@Override
	protected void proc_Msg_Record_Stoped() throws SpeechError, IOException, InterruptedException{
		if( mIsStartFound ){
			mPerflog.pushSessionInfo(SessPerflog.KEY_APP_LAU);
			// upload end flag
			mSession.pushEndFlag();
			// 最低优先级消息，会被自动清除
			updateTimeoutMsg();
		}else{
			DebugLog.LogD( "exit with no speech audio" );
			exit( null );
		}//end of if-else start found
	}//end of function proc_Msg_Record_Stoped
	
	private void startSession() throws Exception{
		if( null!=SpeechUtility.getUtility() ){
			mPerflog.pushSessionInfo(SessPerflog.KEY_APP_SSB);
			DebugLog.LogD( "begin session" );
			proc_Msg_Session_Begin();
		}else{
			DebugLog.LogE( "not init while begin session" );
			exit( new SpeechError(ErrorCode.ERROR_LOGIN) );
		}
	}
	
	@Override
	protected void uploadData(byte[] data,boolean needvadNotify) throws SpeechError{
		// 第一次向msc写入音频时间
		if (false == mFirstDataWrite) {
			mFirstDataWrite = true;
			mPerflog.pushSessionInfo(SessPerflog.KEY_APP_FAU);
			//通知外部开始有音频数据
			if(null != mListener)
			{
				mListener.onEvent(SpeechEvent.EVENT_SPEECH_START, 0, 0, null);
			}
		}

		mSession.pushAudioData(data, data.length);
	}
	
	protected void proc_Msg_Record_Data(Message msg) throws Exception{

		byte[] data = (byte[])msg.obj;
		if (data == null || data.length == 0){
			return;
		}
		
		data = detectAudioData( data );
		
		if( null != data ){
			// 保存录音数据，用以再次识别和保存
			mRecordBuffer.add(data);
			
			uploadData(data, true);
		}//end of if data not null
	}
	
	private byte[] detectAudioData( final byte[] data ) throws Exception{
		byte[] audio = null;
		
		if( null != mDetector ){
			DetectorResult dctResult = null;
			int offset=0;
			int startOffset = 0;
			boolean isStart = false;
			int len = Math.min(AudioDetector.MAX_BUF_LEN, data.length);
			while( 0< len ){
				dctResult=mDetector.detect( data, offset, len, false );
				
				if( ErrorCode.SUCCESS != dctResult.error ){
					throw new SpeechError( dctResult.error );
				}
				
				if( DetectorResult.STATUS_BOS == dctResult.status ){
					mDetector.reset();
				}else if( DetectorResult.STATUS_OK != dctResult.status ){
					if( !mIsStartFound ){
						// 记录偏移
						startOffset = Math.max( startOffset, offset-mLastBuf.length );
						
						mIsStartFound = true;
						isStart = true;
						DebugLog.LogD( "detectAudioData find start and begin session" );
						startSession();
					}
					
					 if( DetectorResult.STATUS_START != dctResult.status ){
						 break;
					 }
					
				}//end of if-else status
				
				offset += len;
				len = Math.min(AudioDetector.MAX_BUF_LEN, data.length-offset);
			}//end of while
			
			if( null != dctResult ){
				callbackVolume( data,  dctResult.volume );
			}
			
			// find start or not
			if( null!=dctResult && mIsStartFound ){
				if( isStart ){
					//保证找到前端点前的 BUF_TIME 的音频，也被保留进识别
					final byte[] tmpBuf = new byte[data.length-startOffset+mLastBuf.length];
					if( mLastBuf.length <= startOffset ){
						System.arraycopy( data, startOffset-mLastBuf.length, tmpBuf, 0, tmpBuf.length );
					}else{
						System.arraycopy( mLastBuf, startOffset, tmpBuf, 0, mLastBuf.length-startOffset );
						System.arraycopy( data, 0, tmpBuf, mLastBuf.length-startOffset, data.length );
					}//end of if-else offset >= mLastBuf.length
					
					audio = tmpBuf;
				}else{
					audio = data;
				}//end of if-else is start
				
				if( DetectorResult.STATUS_EOS==dctResult.status
						|| DetectorResult.STATUS_TIMEOUT==dctResult.status ){
					// 找到后端点
					DebugLog.LogD( "detectAudioData find eos or timeout" );
					vadEndCall();
				}
				
			}else{
				//若未找到，则保存最后的 BUF_TIME 的音频到 mLastBuf
				final int bufLen = Math.min( mLastBuf.length, data.length );
				System.arraycopy( mLastBuf, bufLen, mLastBuf, 0, mLastBuf.length-bufLen );
				System.arraycopy( data, data.length-bufLen, mLastBuf, mLastBuf.length-bufLen, bufLen );
				audio = null;
			}//end of if-else find start
		}else{
			throw new SpeechError( ErrorCode.ERROR_LOCAL_NO_INIT );
		}//end of if-else mDetector not null
		
		return audio;
	}//end of function
}
