package com.iflytek.cloud.msc.isv;

import java.io.UnsupportedEncodingException;
import java.util.Date;

import android.content.Context;
import android.os.SystemClock;

import com.iflytek.cloud.ErrorCode;
import com.iflytek.cloud.SpeechConstant;
import com.iflytek.cloud.SpeechError;
import com.iflytek.cloud.msc.module.MscHandler;
import com.iflytek.cloud.msc.module.MscSession;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.msc.util.log.PerfLogger;
import com.iflytek.cloud.param.ParamBuilder;
import com.iflytek.msc.MSC;
import com.iflytek.msc.MSCSessionInfo;

/**
 * 声纹会话的JNI交互类
 * @author yjzhao
 *
 */
public class VerifySession extends MscSession{

	private final static int ISV_AUDIO_SAMPLE_CONTINUE = 0x02;
	private final static int ISV_AUDIO_SAMPLE_LAST = 0x04;

	private MSCSessionInfo		mSessionOut	= new MSCSessionInfo();
	private MSCSessionInfo		mWriteout	= new MSCSessionInfo();
	private MSCSessionInfo		mModelOut	= new MSCSessionInfo();
	/**
	 * extra vidlist
	 * @return success or error code
	 * @author yjzhao
	 */
	@Override
	public int sessionBegin(Context context,
			String vidlist, MscHandler handler) throws SpeechError,
			UnsupportedEncodingException {
		String param = ParamBuilder.composeIsvSessionParam(context,handler);
		long d1 = SystemClock.elapsedRealtime();
		PerfLogger.appendInfo(PerfLogger.MSC_SESSION_BIGNE, null);
		DebugLog.LogD("QISVSessionBegin begin");
		mClientID = MSC.QISVSessionBegin(param.getBytes(handler.getParamEncoding()),
				vidlist == null ? null : vidlist.getBytes(handler.getParamEncoding()), mSessionOut);
		PerfLogger.appendInfo(PerfLogger.SESSION_BEGIN_END, null);
		
		DebugLog.LogD("QISVSessionBegin ret: " + mSessionOut.errorcode
				+ " time:" + (SystemClock.elapsedRealtime() - d1));
		int errorCode = mSessionOut.errorcode;
		if (0 != errorCode) {
			//增加防止10109、10132错误直接返回
			if(errorCode != ErrorCode.MSP_ERROR_CREATE_HANDLE
					&& errorCode != ErrorCode.MSP_ERROR_OVERFLOW && errorCode != ErrorCode.MSP_ERROR_INVALID_OPERATION)
				throw new SpeechError(errorCode);
		}
		return 0;
	}
	@Override
	public void sessionEnd(String reason) {
		if (null == mClientID){
			return;
		}
		DebugLog.LogD("isv sessionEnd enter ");
		long d1 = System.currentTimeMillis();
		boolean bRet = 0 == MSC.QISVSessionEnd(mClientID,
				reason.getBytes());
		DebugLog.LogD("isv sessionEnd leave:" + bRet + " time:"
						+ (System.currentTimeMillis() - d1));
		mClientID = null;
		mSessionID = null;
	}

	synchronized public void pushAudioData(final byte[] waveData, int waveLen) 
	throws SpeechError{
		writeData(waveData, waveLen, ISV_AUDIO_SAMPLE_CONTINUE);
	}
	
	synchronized public void pushEndFlag() 
	throws SpeechError{
		byte[]  data = new byte[0];
		PerfLogger.appendInfo(PerfLogger.LAST_DATA_FLAG, null);
		DebugLog.LogD( "IsvSession pushEndFlag" );
		writeData(data, 0, ISV_AUDIO_SAMPLE_LAST);
	}

	/**
	 * upload data to server
	 * 
	 * @param waveData 写入的wav 数据
	 * @param waveLen 数据长度
	 * @param audioStatus 音频状态
	 * @throws SpeechError MSC error类型
	 */

	synchronized private void writeData(final byte[] waveData, int waveLen, int audioStatus) 
	throws SpeechError{
		int nRet = MSC.QISVAudioWrite(mClientID,null, waveData, waveLen, audioStatus, mSessionOut);

		DebugLog.LogI("QISVAudioWrite error:" + nRet);
		if (0 != nRet)
			throw new SpeechError(nRet);
	}

	/**
	 * 
	 * @return true if find record end point
	 */
	synchronized public boolean getVadEnd() {
		int eps = mSessionOut.epstatues;
//		DebugLog.LogD("Vad Epstatus=" + eps);
		return eps >= 3;
	}
	/**
	 * get audio data volume value
	 * 
	 * @return
	 */
	synchronized public int getAudioVolume() {
		int ret = 0;
		int vadret = 0;
		try {
			vadret = MSC.QISVGetParam(mClientID, "volume".getBytes(),
					mWriteout);
			if (vadret == 0) {
				String volume = new String(mWriteout.buffer);
				ret = Integer.parseInt(new String(volume));
			} else {
				DebugLog.LogI("VAD CHECK FALSE");
			}
		} catch (Exception e) {
			DebugLog.LogI("getAudioVolume Exception vadret = " + vadret);
		}
		return ret;
	}
	/**
	 * if engine has result,return true
	 * 
	 * @return
	 */
	synchronized public boolean hasResult() {
		return mSessionOut.rsltstatus == 0;
	}


	private byte[] mResult = null;
	public byte[] getResultData()
	{
		return mResult;
	}
	/**
	 * get audio data volume value
	 * 
	 * @return
	 */
	synchronized public String getStringValue(String tag) {
		if(mClientID == null)
			return null;
		int code = 0;
		try {
			//没有方法获取sid
			code = MSC.QISVGetParam(mClientID, tag.getBytes(),mSessionOut);
			if (code == 0) {
				return new String(mSessionOut.buffer);
			} else {
			}
		} catch (Exception e) {
		}
		return null;
	}
	public ResultStatus getStatus() throws SpeechError{

		Date d1 = new Date();
		mResult = MSC.QISVGetResult(mClientID, null, mSessionOut);
		Date d2 = new Date();
		DebugLog.LogI("QISVGetResult leavel:" + (null != mResult) + " time:"
						+ (d2.getTime() - d1.getTime()));

		int error = mSessionOut.errorcode;
		if (error == 0) {
			int resultStatus = mSessionOut.rsltstatus;
			switch (resultStatus) {
			case 1:
				DebugLog.LogD("ResultStatus: noResult" + resultStatus);
				throw new SpeechError(ErrorCode.ERROR_NO_MATCH);
			case 0:
			case 5:
				if(mResult != null)
				{
					DebugLog.LogD("ResultStatus: hasResult" + resultStatus);
					return ResultStatus.hasResult;
				}
			}
			return ResultStatus.noResult;
		}
		DebugLog.LogE("Result: error errorcode is " + error);
		throw new SpeechError(error);
	}
	/**
	 * 获得服务端ID
	 */
	protected String getSessionID()
	{
		if(null == mSessionID )
			mSessionID = getStringValue(TAG_SID);
		return mSessionID;
	}

	/**
	 * 声纹模型操作 包括查询、删除
	 * 
	 * @return
	 * @throws SpeechError
	 */
	public int sendRequest(Context context, MscHandler handler)
			throws UnsupportedEncodingException, SpeechError {
		long d1 = SystemClock.elapsedRealtime();
		String vidlist = handler.getParam().getString(SpeechConstant.ISV_VID);
		String param = ParamBuilder.composeIsvSessionParam(context, handler);
		DebugLog.LogD("sendRequest enter ");

		PerfLogger.appendInfo(PerfLogger.LAST_DATA_FLAG, null);
		char[] queDelSessionID = MSC.QISVQueDelModel(((vidlist == null) ? null
				: vidlist.getBytes(handler.getParamEncoding())), param
				.getBytes(handler.getParamEncoding()), mModelOut);
		PerfLogger.appendInfo(PerfLogger.GET_RESULT, null);
		
		// 操作完模型后一定要释放资源，否则会报10132错误
		MSC.QISVQueDelModelRelease(queDelSessionID);

		int ret;
		if (mModelOut.errorcode != ErrorCode.SUCCESS) {
			ret = mModelOut.errorcode;
		} else {
			String result = new String(mModelOut.buffer);
			ret = "true".equals(result) ? ErrorCode.SUCCESS
					: ErrorCode.MSP_ERROR_FAIL;
		}
		
		DebugLog.LogD("sendRequest leave:" + ret + " time:"
				+ (SystemClock.elapsedRealtime() - d1));

		if (ErrorCode.SUCCESS != ret && ErrorCode.MSP_ERROR_FAIL != ret) {
			throw new SpeechError(ret);
		}

		return ret;
	}
}
