package com.iflytek.cloud.msc.ivw;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map.Entry;

import com.iflytek.cloud.ErrorCode;
import com.iflytek.cloud.Setting;
import com.iflytek.cloud.SpeechConstant;
import com.iflytek.cloud.SpeechError;
import com.iflytek.cloud.SpeechEvent;
import com.iflytek.cloud.msc.ivw.aimic.AIMic;
import com.iflytek.cloud.msc.module.MscSession;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.param.HashParam;
import com.iflytek.cloud.param.MscKeys;
import com.iflytek.cloud.param.MscKeys.VadStatus;
import com.iflytek.cloud.record.PcmRecorder;
import com.iflytek.cloud.param.ParamBuilder;

import android.content.Context;
import android.os.Bundle;
import android.os.HandlerThread;
import android.os.Message;
import android.text.TextUtils;

public class AimicWakeuper extends MscWakeuper implements AIMic.IvwDataListener, AIMic.AsrDataListener {
	private final String KEY_IVW_CALLER = "ivw_caller";
	
	private final int MSG_WAKEUP_MSG = MSG_END + 1;
	private final int MSG_AIMIC_START = MSG_END + 2;
	
	private final String KEY_CB_SID = "sid";
	private final String KEY_CB_MSG = "msg";
	private final String KEY_CB_ARG1 = "arg1";
	private final String KEY_CB_ARG2 = "arg2";
	private final String KEY_CB_RESULT = "result";

	private int mChannelNum = MscKeys.IVW_CHANNEL_SINGLE;
	
	private AIMic mAIMic = null;
	private final int AIMIC_START_DELAY = 0;

	public AimicWakeuper(Context context, HashParam param, HandlerThread mHandlerThread) {
		super(context, param, mHandlerThread);
		
		mChannelNum = getParam().getInt( SpeechConstant.IVW_CHANNEL_NUM, MscKeys.IVW_CHANNEL_SINGLE );
		getParam().putParam( KEY_IVW_CALLER, "1", false ); 	//多路唤醒必须设置 的MSC参数
		
		mAIMic = AIMic.getAIMic();
		if( null == mAIMic ){
			mAIMic = AIMic.createAIMic( this.getParam().remove( MscKeys.AIMIC_INIT_PARAM ) );
		}
		
		if( null != mAIMic ){
			mAIMic.registerListener( (AIMic.IvwDataListener)this );
			mAIMic.registerListener( (AIMic.AsrDataListener)this );
		}
	}
	
	/*************************************  begin message process  ******************************************/
	@Override
	protected void onMsgProcess(Message msg) throws Throwable, SpeechError {
		super.onMsgProcess(msg);

		switch (msg.what) {
		case MSG_WAKEUP_MSG:
			proc_Wakeup_Msg(msg);
			break;
			
		case MSG_AIMIC_START:
			proc_Msg_AIMicStart();
			break;

		default:
			break;
		}
	}

	private void proc_Wakeup_Msg(Message msg) throws SpeechError, Throwable{
		DebugLog.LogD( "proc_Wakeup_Angle enter" );
		
		SpeechError error = null;
		if (mListener != null) {
			final Bundle extra = msg.peekData();
			final int ivwStatus = extra.getInt( KEY_CB_MSG );
			IvwMessage ivwMsg = null;
			try{
				ivwMsg = IvwMessage.values()[ ivwStatus ];
			}catch( Throwable e ){
				DebugLog.LogE( "warn: unmatched ivw message: "+ivwStatus );
				ivwMsg = IvwMessage.undefined_0;
			}//end of try-catch
			
			final int arg1 = extra.getInt( KEY_CB_ARG1 );
			
			switch( ivwMsg ){
			case IVW_MSG_WAKEUP:
				onWakeup( extra );
				break;
			case IVW_MSG_ERROR:
				error = new SpeechError( arg1  );
				break;
			case IVW_MSG_ISR_RESULT:
				Message msgRecog = obtainMessage( MSG_RESULT, arg1, REC_RESULT, extra.getByteArray(KEY_CB_RESULT) );
				Priority priority = hasMessages(MSG_RESULT) ? Priority.normal : Priority.max;	//消息队列在执行很耗时的任务 后至的结果数据被插入队列前，导致之前的结果无法展示
				sendMsg( msgRecog, priority, false, 0 );
				break;
			case IVW_MSG_ISR_EPS:
				if ( VadStatus.VAD_EOS.ordinal() == arg1 ) {
					vadEndCall();
				}
				break;
			case IVW_MSG_VOLUME:
				if( null != mListener ){
					mListener.onVolumeChanged( arg1 );
				}
				break;
			case IVW_MSG_ENROLL:
				Message enrollMsg = obtainMessage( MSG_RESULT, 0, ENROLL_RESULT, extra.getByteArray(KEY_CB_RESULT) );
				sendMsg( enrollMsg, Priority.max, false, 0 );
				break;
			case IVW_MSG_RESET:
				DebugLog.LogD( "proc_Wakeup_Msg reset msg" );
				break;
				default:	//do nothing
			}//end of switch ivwMsg
		}else{
			DebugLog.LogE( "proc_Wakeup_Angle error: listener is null" );
			error = new SpeechError( ErrorCode.ERROR_UNKNOWN );
		}//end of if-else listener not null
		
		if( null != error ){
			DebugLog.LogE( "wakeup msg error: "+error.getErrorCode() );
			throw error;
		}//end of if error not null
	}
	
	@Override
	protected void proc_Msg_Session_Begin() throws SpeechError, Throwable{
		AIMicBegin();
		setStatus(Status.recording);
		
		if( null != this.mListener ){
			this.mListener.onEvent( SpeechEvent.EVENT_SESSION_BEGIN, 0, 0, null );
		}
		
		sendMsg( MSG_AIMIC_START, Priority.normal, true, AIMIC_START_DELAY );
	}//end of function proc_Msg_Session_Begin
	
	/**
	 * 启动消息处理
	 * @throws Exception
	 */
	@Override
	protected void proc_Msg_Start() throws Exception {	
		DebugLog.LogD("start connecting");
		//初始化唤醒状态
		isWake = false;
		int read_rate = getParam().getInt(MscKeys.KEY_RECORD_READ_RATE, PcmRecorder.READ_INTERVAL40MS);

		// 判断不是外部写录音会话
		if( MscKeys.VALUE_AUDIO_AIMIC == mAudioSource ){
			if( null == mAIMic ){
				DebugLog.LogE( "create AIMIC failed!" );
				throw new SpeechError( ErrorCode.ERROR_ENGINE_INIT_FAIL );
			}
		}else if(mAudioSource != MscKeys.VALUE_AUDIO_EXTERNAL && isRunning()){
			DebugLog.LogD( "start  record");
			if (mRecorder == null ){ 
				mRecorder = new PcmRecorder(getSampleRate(), read_rate,mAudioSource);
				mRecorder.startRecording(this);
			}
		}
		
		// 插入到消息头部，防止音频被预先处理
		sendMsg(MSG_SESSION_BEGIN,Priority.max,false,0);
	}
	
	private void proc_Msg_AIMicStart() throws Throwable{
		if( isRunning() ){
			startAIMicListening();
			
			if (getStatus() != Status.exiting && null != mListener){
				mListener.onBeginOfSpeech();
			}
		}else{
			DebugLog.LogD( "It's not running while start aimic listening!" );
		}
	}
	/*************************************  end message process  ******************************************/
	
	@Override
	protected void onEnd(SpeechError error) {
		DebugLog.LogD( "Aimic Wakeuper onEnd enter" );
		
		error = AIMicEnd( error );
		
		super.onEnd( error );
	}
	
	@Override
	public int writeAudio(byte[] data,int offset,int length) {
		int ret = ErrorCode.ERROR_LOCAL_NO_INIT;
		
		if( null != mAIMic ){
			ret = mAIMic.writeAudio( data, offset, length );
		}//end of mAIMic not null
		
		return ret;
	}
	
	/*************************************  begin AIMIC.Listener implements  ******************************************/
	@Override
	public void onWakeupAudio(byte[] audioData, int dataLen, int arg1, Object arg2) {
	}
	
	@Override
	public void onWakeupMsg(int msg, int arg1, int arg2, byte[] sid, int sidLen, byte[] result, int resultLen, byte[] audio, int audioLen ) {
//		DebugLog.LogD( "onWakeupMsg enter" );
		
		try{
			final Bundle extra = new Bundle();
			
			//直接在回调中获取音频
			byte[] data = null;
			if( IvwMessage.IVW_MSG_WAKEUP.ordinal()==msg  ){
				data = new byte[ audioLen ];
				System.arraycopy( audio, 0, data, 0, data.length );
				DebugLog.LogD( "onWakeupMsg audio length:"+data.length );
			}
			
			extra.putByteArray( KEY_WAKEUP_AUDIO, data );
			
			if( null != sid ){
				byte[] sidBuf = new byte[ sidLen ];
				System.arraycopy( sid, 0, sidBuf, 0, sidBuf.length );
				extra.putByteArray( KEY_CB_SID, sidBuf );
			}
			
			if( null != result ){
				byte[] resultBuf = new byte[ resultLen ];
				System.arraycopy( result, 0, resultBuf, 0, resultBuf.length );
				extra.putByteArray( KEY_CB_RESULT, resultBuf );
			}
			extra.putInt( KEY_CB_MSG, msg );
			extra.putInt( KEY_CB_ARG1, arg1 );
			extra.putInt( KEY_CB_ARG2, arg2 );
			
			
			final Message ivwMsg = obtainMessage( MSG_WAKEUP_MSG );
			ivwMsg.setData( extra );
			final Priority priority = hasMessages(MSG_WAKEUP_MSG) ? Priority.normal : Priority.max;	//消息队列在执行很耗时的任务 后至的结果数据被插入队列前，导致之前的结果无法展示
			sendMsg( ivwMsg, priority, false, 0 );
		}catch( Throwable e ){
			DebugLog.LogE( e );
			System.gc();
			errorExit( new SpeechError(ErrorCode.ERROR_UNKNOWN) ) ;
		}//end of try-catch
		
//		DebugLog.LogD( "onWakeupMsg leave" );
	}

	public void onError( final int errorCode ){
		this.onError( new SpeechError(errorCode) );
	}
	/*************************************  end AIMIC.Listener implements  ******************************************/
	
	/*************************************  begin utils  ******************************************/
	private void AIMicBegin() throws SpeechError, Throwable{
		int status = ErrorCode.SUCCESS;
		
		if( 0 >= mChannelNum ){
			DebugLog.LogE( "Channel number "+mChannelNum+" is less than 1 !" );
			throw new SpeechError( ErrorCode.ERROR_INVALID_PARAM );
		}
		
		if( null == mAIMic ){
			DebugLog.LogE( "create AIMIC failed!" );
			throw new SpeechError( ErrorCode.ERROR_ENGINE_INIT_FAIL );
		}
		
		mAIMic.setShowLog( Setting.getShowLog(), Setting.getLogLevel().ordinal() );
		
		if( ErrorCode.SUCCESS == status ){
			status = mAIMic.reset();
			
			///////////////////////////////////////////////////////////////////////////////
			HashMap<String, String> key2Vals = this.getParam().getHash();
			DebugLog.LogD( "AIMicBegin param count: "+key2Vals.size() );
			int i = 0;
			for( Iterator<Entry<String, String>> iterator = key2Vals.entrySet().iterator()
					; iterator.hasNext() && ErrorCode.SUCCESS==status
					; ){
				final Entry<String, String> entry = iterator.next();
				if( entry.getKey().startsWith(MscKeys.AIMIC_SSB_PARAM_PREFIX) ){
					final String realAIMicKey = entry.getKey().substring( MscKeys.AIMIC_SSB_PARAM_PREFIX.length() );
					status = mAIMic.setParameter( realAIMicKey, entry.getValue() );
					
					iterator.remove();
				}
				
				++i;
			}//end of for
			DebugLog.LogD( "AIMicBegin for count: "+i );
			//////////////////////////////////////////////////////////////////////////////////////////////////
			
			if( ErrorCode.SUCCESS == status ){
				status = mAIMic.setParameter( AIMic.KEY_IVW_THREAD_NUM, Integer.toString(mChannelNum) );
			}
			
			if( ErrorCode.SUCCESS==status ){
//				AIMICSetParam 中调用了 IVW SSB
				status = mAIMic.setParameter( AIMic.KEY_AIMIC_IVW_SSB
						, ParamBuilder.composeIvwSessionParam(this.mContext, session_type, this) );
			}//end of if AIMICResetEng succeed
		}//end of if aimic is valid
		
		if( ErrorCode.SUCCESS != status ){
			DebugLog.LogE( "aimic init error: "+status );
			throw new SpeechError( status );
		}
	}
	
	private SpeechError AIMicEnd( SpeechError error ){
		DebugLog.LogD( "AIMicEnd enter" );
		
		if( null != mAIMic ){
			mAIMic.unregisterListener( (AIMic.IvwDataListener)this );
			mAIMic.unregisterListener( (AIMic.AsrDataListener)this );
			mAIMic.stopListening();
			
			Throwable cause = null;
			int ret = ErrorCode.SUCCESS;
			try{
				String strReason = "success";
				if(mUserCancel){
					strReason = "user abort";
				}else if(error != null){
					strReason = "error" + error.getErrorCode();
				}
				
				ret = mAIMic.setParameter( AIMic.KEY_AIMIC_IVW_SSE, strReason );
			}catch( UnsatisfiedLinkError e ){
				cause = e;
				ret = ErrorCode.ERROR_UNSATISFIED_LINK;
			}catch( Throwable e ){
				cause = e;
				ret = ErrorCode.ERROR_UNKNOWN;
			}//end of try-catch
			
			if( null==error && (null!=cause || ErrorCode.SUCCESS!=ret) ){
				DebugLog.LogE( "AIMicEnd error!" );
				error = new SpeechError( cause, ret );
			}//end of if error null
		}//end of if aimic valid
		
		DebugLog.LogD( "AIMicEnd leave" );
		return error;
	}//end of function AIMicAnd

	protected void onWakeup( final Bundle bundle ) throws SpeechError, Throwable{
		// 唤醒结果回调消息，优先级最高
		isWake = true;
		MscSession.ResultStatus status = MscSession.ResultStatus.resultOver;
		if( keep_alive || MscKeys.IVW_ONESHOT.equals(session_type) ){
			status = MscSession.ResultStatus.hasResult;
		}
		
		Message resultMsg = obtainMessage( MSG_RESULT, status.ordinal(), IVW_RESULT, bundle.getByteArray(KEY_CB_RESULT) );
		resultMsg.setData( bundle );

		Priority priority = hasMessages(MSG_RESULT) ? Priority.normal : Priority.max;	//消息队列在执行很耗时的任务 后至的结果数据被插入队列前，导致之前的结果无法展示
		sendMsg( resultMsg, priority, false, 0 );
	}
	
	private void errorExit( SpeechError error ){
		exit( error );
	}
	
	private void startAIMicListening() throws SpeechError{		
		int ret = ErrorCode.SUCCESS;
		final String alsaCard = this.getParam().remove( AIMic.KEY_ALSA_CARD );
		if( !TextUtils.isEmpty(alsaCard) ){
			ret = mAIMic.setParameter( AIMic.KEY_ALSA_CARD, alsaCard );
		}
		
		final String alsaSampleRate = this.getParam().remove( AIMic.KEY_ALSA_SAMPLE_RATE );
		if( ErrorCode.SUCCESS==ret && !TextUtils.isEmpty(alsaSampleRate) ){
			ret = mAIMic.setParameter( AIMic.KEY_ALSA_SAMPLE_RATE, alsaSampleRate );
		}
		
		final String saveAlsa = this.getParam().remove( AIMic.KEY_SAVE_ALSA );
		if( ErrorCode.SUCCESS==ret && !TextUtils.isEmpty(saveAlsa) ){
			ret = mAIMic.setParameter( AIMic.KEY_SAVE_ALSA, saveAlsa );
		}
		
		if( ErrorCode.SUCCESS == ret ){
			final int aimicAudioSource = MscKeys.VALUE_AUDIO_AIMIC==this.mAudioSource ? AIMic.AUDIO_ALSA : AIMic.AUDIO_EXTERNAL;
			ret = mAIMic.setParameter( AIMic.KEY_AUDIO_SOURCE, Integer.toString(aimicAudioSource) );
		}
		
		if( ErrorCode.SUCCESS == ret ){
			ret = mAIMic.startListening();
		}
		
		if( ErrorCode.SUCCESS != ret ){
			DebugLog.LogE( "start aimic listening failed: "+ret );
			throw new SpeechError( ret );
		}
	}
	/*************************************  begin utils  ******************************************/

	/*************************************  begin AIMIC.AsrDataListener implements  ******************************************/
	@Override
	public void onRecogAudio(byte[] audio, int audioLen, int arg2, Object arg3) {
		try{
			final Bundle bundle = new Bundle();
			final byte[] buf = new byte[audioLen];
			System.arraycopy( audio, 0, buf, 0, audioLen );
			bundle.putByteArray( SpeechEvent.KEY_EVENT_RECORD_DATA,buf );
			mListener.onEvent( SpeechEvent.EVENT_RECORD_DATA, 0, 0,bundle );
		}catch( Throwable e ){
			DebugLog.LogE( e );
			onError( new SpeechError( e, ErrorCode.ERROR_UNKNOWN) );
		}//end of try-catch
	}
	/*************************************  end AIMIC.AsrDataListener implements  ******************************************/
}