package com.iflytek.cloud.msc.ivw.aimic.impl;

import java.io.File;
import java.io.RandomAccessFile;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;

import com.iflytek.cloud.ErrorCode;
import com.iflytek.cloud.SpeechError;
import com.iflytek.cloud.msc.ivw.aimic.AIMic;
import com.iflytek.cloud.msc.util.DataUtil;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.msc.AIMIC;
import com.iflytek.msc.AIMIC.Listener;

import android.os.Environment;
import android.os.MemoryFile;
import android.text.TextUtils;

public class AIMicImpl extends AIMic{
	public static final String KEY_BUF_MODE = "buf_mode";
	public static final String KEY_ALSA_PERSIZE = "alsa_per_size";
	
	public static final int BUF_MODE_STANDARD = 0;
	public static final int BUF_MODE_STRICT = 1; 
	public static final int BUF_MODE_LOOSE = 2;
	
	private final int PER_BUF_LEN = 512 * 96;
	private final int BUF_LEN = PER_BUF_LEN * 10;
	private final int MAX_BUF_LEN = BUF_LEN * 2;
	
	private static AIMicImpl sInstance = null;
	private static final Object sSync = new Object(); 
	private final Object mSync = new Object();
	private boolean mIsValid = false;
	
	private final byte[] mValueBuffer = new byte[256];
	private final LengthLimitedBuffer mSrcAudios = new LengthLimitedBuffer( BUF_LEN, PER_BUF_LEN, BUF_LEN/2, false, false );
	private final AudioWritingThread mAuwThread = new AudioWritingThread( "AIMicAudioWritingThread" );
	private int mBufMode = BUF_MODE_LOOSE;
	
	private final AIMicListener mAIMicListener = new AIMicListener();
	private int mAsrBufferingTime = 4000;
	
	private final int DEF_ALSA_SAMPLE_RATE = 16000;
	private final int DEF_ALSA_CARD = 2;
	private final int DEF_ALSA_PERSIZE = 1536;
	
	private AudioRecorder mRecorder = null;
	private final AudioRecordListener mAudioReocrdListener = new AudioRecordListener();
	private int mAlsaSampleRate = DEF_ALSA_SAMPLE_RATE;
	private int mAlsaCard = DEF_ALSA_CARD;
	
	private int mAlsaPerSize = DEF_ALSA_PERSIZE;
	
	private int mAudioSource = AUDIO_ALSA;
	private boolean mIsListening = false;
	
	private AIMicImpl( final String params ) throws UnsatisfiedLinkError, SpeechError, Throwable{
		DebugLog.LogD( "aimic constructor enter: "+ params );
		int status = ErrorCode.SUCCESS;
		final StringBuffer paramsBuf = new StringBuffer();
		if( !TextUtils.isEmpty(params) ) {
			paramsBuf.append( params );
		}
		
		if( !AIMIC.isLoaded() ){
			String strLibs = null;
			if( null != params ){
				final int idxOfLibName = params.indexOf(KEY_LIB_NAME);
				if( 0<=idxOfLibName && idxOfLibName<params.length() ){
					final char SEP = ',';
					int idxOfLibNameEnd = params.indexOf( SEP, idxOfLibName );
					if( 0 > idxOfLibNameEnd ){
						idxOfLibNameEnd = params.length();
					}
					
					final char SEP_EQ = '=';
					final int idxOfLibEq = params.indexOf( SEP_EQ, idxOfLibName );
					strLibs = params.substring( idxOfLibEq+1, idxOfLibNameEnd );
					paramsBuf.delete( idxOfLibName, idxOfLibNameEnd );
				}
			}//end of params not null
			
			AIMIC.loadLibrary( strLibs );
		}//end of if aimic not loaded
		
		if( !AIMIC.isValid() ){
			
			final String strAIMicInitParam = paramsBuf.toString();
			status = AIMIC.AIMICNew( DataUtil.getNativeByteArray(strAIMicInitParam), this.mAIMicListener );
			if( ErrorCode.SUCCESS != status ){
				DebugLog.LogE( "AIMICNew return error: "+status );
				AIMIC.AIMICDestroy( AIMIC.getHandler() );
			}
		}//end of if aimic not valid
		
		if( ErrorCode.SUCCESS != status ){
			throw new SpeechError( status );
		}else{
			mIsValid = true;
			mAuwThread.start();
			
			if( BUF_MODE_STANDARD==mBufMode || BUF_MODE_LOOSE==mBufMode ){
				mSrcAudios.raiseMaxLength( MAX_BUF_LEN-BUF_LEN );
			}//end of buf mode
		}
		
		DebugLog.LogD( "aimic constructor leave: "+ status );
	}
	
	public static AIMicImpl getAIMicImpl(){
		DebugLog.LogD( "aimic getAIMic enter" );
		
		AIMicImpl instance = null;
		
		synchronized( sSync ){
			instance = sInstance;
		}
		
		DebugLog.LogD( "aimic getAIMic leave: "+instance );
		return instance;
	}
	
	public static AIMicImpl createAIMicImpl( final String params ){
		DebugLog.LogD( "aimic createAIMic enter" );
		
		AIMicImpl instance = null;
		
		synchronized( sSync ){
			if( null == sInstance ){
				try{
					sInstance = new AIMicImpl( params );
				}catch( SpeechError e ){
					DebugLog.LogE( e );
				}catch( UnsatisfiedLinkError e ){
					DebugLog.LogE( e );
				}catch( Throwable e ){
					DebugLog.LogE( e );
				}// end of try-catch
			}//end of if instance is null
			
			instance = sInstance;
		}//end of synchronized
		
		DebugLog.LogD( "aimic createAIMic leave: "+instance );
		return instance;
	}

	public void destroy(){
		DebugLog.LogD( "aimic destroy enter" );
		
		synchronized( mSync ){
			this.mIsValid = false;
			
			if( null!=mAIMicListener ){
				mAIMicListener.clear();
			}
			
			if( null != mRecorder ){
				mRecorder.stopRecording();
				mRecorder.destroy();
				mRecorder = null;
			}
		}//end of synchronized
		
		synchronized( mAuwThread ){
			if( mAuwThread.isAlive() ){
				mAuwThread.notify();
			}
		}//end of synchronized
		
		synchronized( sSync ){
			if( null != sInstance ){
				try {
					AIMIC.AIMICDestroy( AIMIC.getHandler() );
				} catch (Throwable e) {
					DebugLog.LogE( e );
				}
				
				sInstance = null;
			}//end of if sInstance not null
		}//end of synchronized
		
		System.gc();
		
		DebugLog.LogD( "aimic destroy leave" );
	}
	
	public void registerListener( final IvwDataListener listener ){
		DebugLog.LogD( "aimic  registerListener enter: "+listener );
		
		synchronized( mSync ){
			mAIMicListener.registerListener( listener );
		}//end of synchronized
		
		DebugLog.LogD( "aimic  setParameter leave" );
	}
	
	public void registerListener( final AsrDataListener listener ){
		DebugLog.LogD( "aimic  registerListener enter: "+listener );
		
		synchronized( mSync ){
			mAIMicListener.registerListener( listener );
		}//end of synchronized
		
		DebugLog.LogD( "aimic  registerListener leave" );
	}
	
	public void unregisterListener( final IvwDataListener listener ){
		DebugLog.LogD( "aimic  unregisterListener enter: "+listener );
		
		synchronized( mSync ){
			mAIMicListener.unregisterListener( listener );
			
			if( mAIMicListener.isEmpty() ){
				stopListening();
			}
		}//end of synchronized	
		
		DebugLog.LogD( "aimic  unregisterListener leave" );
	}
	
	public void unregisterListener( final AsrDataListener listener ){
		DebugLog.LogD( "aimic  unregisterListener enter: "+listener );
		
		synchronized( mSync ){
			mAIMicListener.unregisterListener( listener );
			
			if( mAIMicListener.isEmpty() ){
				stopListening();
			}
		}//end of synchronized	
		
		DebugLog.LogD( "aimic  unregisterListener leave" );
	}
	
	public int setParameter( final String key, final String value ){
		DebugLog.LogD( "aimic  setParameter enter key="+key+", value="+value );
		int ret = 0;
		
		try{
			synchronized( mSync ){
				DebugLog.LogD( "aimic  setParameter sync" );
				if( KEY_ASR_BUFFER_TIME.equalsIgnoreCase(key) ){
					this.mAsrBufferingTime = Integer.parseInt( value );
					this.mAIMicListener.setAsrBufferingTime( mAsrBufferingTime );
				}else if( KEY_ALSA_SAMPLE_RATE.equalsIgnoreCase(key) ){
					this.mAlsaSampleRate = Integer.parseInt( value );
				}else if( KEY_ALSA_CARD.equalsIgnoreCase(key) ){
					this.mAlsaCard = Integer.parseInt( value );
				}else if( KEY_ALSA_PERSIZE.equalsIgnoreCase(key) ){
					this.mAlsaPerSize = Integer.parseInt( value );
				}else if( KEY_SAVE_ALSA.equalsIgnoreCase(key) ){
					mAudioReocrdListener.setSaveAlsaData( getBoolean(value) );
				}else if( KEY_AUDIO_SOURCE.equalsIgnoreCase(key) ){
					this.mAudioSource = Integer.parseInt( value );
				}else if( KEY_BUF_MODE.equalsIgnoreCase(key) ){
					this.mBufMode = Integer.parseInt( value );
					switch( mBufMode ){
					case BUF_MODE_STRICT:
						mSrcAudios.raiseMaxLength( BUF_LEN-mSrcAudios.getMaxLength() );
						break;
						
					default:
						mSrcAudios.raiseMaxLength( MAX_BUF_LEN-mSrcAudios.getMaxLength() );
					}//end of switch
					
					if( isValid() ){
						DebugLog.LogD( "aimic  setParameter Native" );
						ret = AIMIC.AIMICSetParam( AIMIC.getHandler()
								, DataUtil.getNativeByteArray(key)
								, DataUtil.getNativeByteArray(value) );
					}else{
						ret = ErrorCode.ERROR_AIMIC_NOT_INIT;
					}//end of if-else valid
				}else if( KEY_THREAD_PRIORITY.equalsIgnoreCase(key) ){
					if( isValid() ){
						DebugLog.LogD( "aimic  setParameter Native" );
						ret = AIMIC.AIMICSetParam( AIMIC.getHandler()
								, DataUtil.getNativeByteArray(key)
								, DataUtil.getNativeByteArray(value) );
						
						final int priority = Integer.parseInt( value );
						this.mAuwThread.setThreadPriority( priority );
					}else{
						ret = ErrorCode.ERROR_AIMIC_NOT_INIT;
					}//end of if-else valid
				}else{
					if( isValid() ){
						DebugLog.LogD( "aimic  setParameter Native" );
						ret = AIMIC.AIMICSetParam( AIMIC.getHandler()
								, DataUtil.getNativeByteArray(key)
								, DataUtil.getNativeByteArray(value) );
					}else{
						ret = ErrorCode.ERROR_AIMIC_NOT_INIT;
					}//end of if-else valid
				}//end of if-else other key
			}//end of synchronized
		}catch( UnsatisfiedLinkError e ){
			DebugLog.LogE( e );
			ret = ErrorCode.ERROR_UNSATISFIED_LINK;
		}catch( Throwable e ){
			DebugLog.LogE( e );
			ret = ErrorCode.ERROR_UNKNOWN;
		}
		
		DebugLog.LogD( "aimic  setParameter leave: "+ret );
		return ret;
	}
	
	public String getParameter( final String key ){
		DebugLog.LogD( "aimic  getParameter enter" );
		
		String value = null;
		byte[] buffer = mValueBuffer;
		
		synchronized( mSync ){
			if( isValid() ){
				try{
					int ret = AIMIC.AIMICGetParam( AIMIC.getHandler(), DataUtil.getNativeByteArray(key), buffer );
					if( ErrorCode.SUCCESS == ret ){
						value = DataUtil.getStringFromNativeByteArray( buffer );
					}else{
						DebugLog.LogE( "get parameter error: "+ret );
					}//end of if-else ret success
				}catch( Throwable e ){
					DebugLog.LogE( e );
				}//end of try-catch
			}
		}//end of synchronized
		
		DebugLog.LogD( "aimic  getParameter leave: "+ value );
		return value;
	}
	
	public int reset(){
		DebugLog.LogD( "aimic  reset enter" );
		int status = 0;
		
		synchronized( mSync ){
			try{
				if( isValid() ){
					status = AIMIC.AIMICResetEng( AIMIC.getHandler() );
				}else{
					status = ErrorCode.ERROR_AIMIC_NOT_INIT;
				}
			}catch( SpeechError e ){
				DebugLog.LogE( e );
				status = e.getErrorCode();
			}catch( UnsatisfiedLinkError e ){
				DebugLog.LogE( e );
				status = ErrorCode.ERROR_UNSATISFIED_LINK;
			}catch( Throwable e ){
				DebugLog.LogE( e );
				status = ErrorCode.ERROR_UNKNOWN;
			}// end of try-catch
		}//end of synchronized
		
		DebugLog.LogD( "aimic  reset leave: "+status );
		return status;
	}
	
	public int startListening(){
		DebugLog.LogD( "aimic  startListening enter" );
		
		int ret = ErrorCode.ERROR_AIMIC_NOT_INIT;
		
		synchronized( mSync ){
			if( !mIsListening ){
				DebugLog.LogD( "aimic  startListening clear old datas." );
				mSrcAudios.clear();
			}
			
			if( isValid() ){
				if( AUDIO_ALSA == mAudioSource ){
					ret = startAudioRecord();
				}else if( AUDIO_EXTERNAL == mAudioSource ){
					if( null != mRecorder ){
						ret = ErrorCode.ERROR_AIMIC_BUSY;
						DebugLog.LogE( "startListening failed, current internal recorder is not stoped!" );
					}else{
						ret = ErrorCode.SUCCESS;
					}//end of if-else is listening
				}else{
					ret = ErrorCode.ERROR_AIMIC_INVALID_PARA_VALUE;
					DebugLog.LogE( "startListening failed, invalid audio source: "+mAudioSource );
				}//end of if-else audio source
			}//end of if is valid
			
			mIsListening = ErrorCode.SUCCESS==ret;
		}//end of synchronized
		
		DebugLog.LogD( "aimic  startListening leave: "+ret );
		return ret;
	}
	
	public void stopListening(){
		DebugLog.LogD( "aimic  stopListening enter" );
		
		synchronized( mSync ){
			if( null==mAIMicListener || mAIMicListener.isEmpty() ){
				DebugLog.LogD( "AIMic Listener is empty, audio recorder will stop recording." );
				
				stopAudioRecord();
				mIsListening = false;
			}
		}//end of synchronized
		
		DebugLog.LogD( "aimic  stopListening leave" );
	}
	
	public boolean isListening(){
		DebugLog.LogD( "aimic  isListening enter" );
		boolean status = false;
		synchronized( mSync ){
			status = mIsListening;
		}//end of synchronized
		
		DebugLog.LogD( "aimic  isListening leave ret: "+status );
		
		return status;
	}
	
	public int writeAudio( final byte[] audioData, final int offset, final int dataLen ){
		int ret = ErrorCode.ERROR_UNKNOWN;
		
		synchronized( mSync ){
			if( isValid() ){
				ret = pushAudio( audioData, offset, dataLen );
			}else{
				ret = ErrorCode.ERROR_AIMIC_NOT_INIT;
				DebugLog.LogE( "write audio while not init!" );
			}
		}//end of synchronized
		
		return ret;
	}
	
	public static String getVesion(){
		String version = null;
		synchronized( sSync ){
			try{
				byte[] btVesion = AIMIC.AIMICGetVersion();
				version =  DataUtil.getStringFromNativeByteArray( btVesion );
			}catch( Throwable e ){
				DebugLog.LogE( e );
			}// end of try-catch
		}//end of synchronized
		return version;
	}
	
	public void setShowLog( final boolean enable, final int level ){
		synchronized( mSync ){
			try {
				DebugLog.setShowLog( enable );
				DebugLog.setLogLevel( DebugLog.LOG_LEVEL.values()[level] );
				AIMIC.AIMICDebugLog( enable, level );
				if( null != mRecorder ){
					mRecorder.setShowLog( enable && DebugLog.LOG_LEVEL.detail.ordinal() >= level );
				}
			} catch (Throwable e) {
				DebugLog.LogE( e );
			}
		}//end of synchronized
	}
	
	public static int getChannel(){
		int ret = -1;
		
		try{
			ret = AIMIC.AIMICGetChannel();
		}catch( Throwable e ){
			DebugLog.LogE( e );
		}
		
		return ret;
	}
	
	private boolean isValid(){
		return mIsValid;
	}
	
	private int pushAudio( final byte[] audioData, final int offset, final int dataLen ){
		int ret = ErrorCode.ERROR_AIMIC_BUSY;
		
		try{
			mSrcAudios.put( audioData, offset, dataLen );
			
			synchronized( mAuwThread ){
				if( Thread.State.WAITING==mAuwThread.getState() ){
					mAuwThread.notify();
				}
			}//end of synchronized
			
			final SpeechError error = mAuwThread.getError();
			if( null != error ){
				ret = error.getErrorCode();
				mAuwThread.resetError();
			}else{
				ret = ErrorCode.SUCCESS;
			}
		}catch( OutOfMemoryError e ){
			DebugLog.LogE( e );
			if( BUF_MODE_LOOSE == mBufMode ){
				DebugLog.LogE( "write audio too soon, current audios  in buffer will be ignored!" );
				mSrcAudios.clear();
				ret = ErrorCode.SUCCESS;
			}else{
				DebugLog.LogE( "write audio too soon, please wait for a second, and try again!" );
				ret = ErrorCode.ERROR_AIMIC_BUSY;
			}
		}catch( IllegalArgumentException e ){
			DebugLog.LogE( e );
			 ret = ErrorCode.ERROR_AIMIC_INVALID_PARA;
		}catch( NullPointerException e ){
			DebugLog.LogE( e );
			 ret = ErrorCode.ERROR_AIMIC_NULL_HANDLE;
		}catch( Throwable e ){
			DebugLog.LogE( "write audio too soon, please wait for a second, and try again!" );
			DebugLog.LogE( e );
			 ret = ErrorCode.ERROR_UNKNOWN;
		}
		
		return ret;
	}
	
	private boolean getBoolean( final String value ){
		return  null!=value && ( value.equals("true" ) || value.equals( "1" ) );
	}
	
	private int startAudioRecord(){
		int ret = ErrorCode.ERROR_AIMIC_NOT_INIT;
		
		if( isValid() ){
			mRecorder = AudioRecorder.getInstance();
			if( null == mRecorder
					|| mRecorder.getCardDevId() != mAlsaCard 
					|| mRecorder.getSampleRate() != mAlsaSampleRate
					|| mRecorder.getPeriodSize() != mAlsaPerSize ){
				if( null != mRecorder ){
					mRecorder.stopRecording();
					mRecorder.destroy();
					mRecorder = null;
				}
				
				DebugLog.LogD( "create new audio recorder" );
				mRecorder = AudioRecorder.createInstance( mAlsaCard, mAlsaSampleRate, mAlsaPerSize );
			}//end of if recorder is null or need recreate
			
			if( null != mRecorder ){
				if( mRecorder.isRecording() 
						&& !mAudioReocrdListener.equals(mRecorder.getListener()) ){
					DebugLog.LogD( "Current record listener is not this, recorder will be stoped." );
					mRecorder.stopRecording();
				}
				
				mRecorder.setShowLog( DebugLog.getShowLog() && DebugLog.LOG_LEVEL.detail.ordinal() >= DebugLog.getLogLevel().ordinal() );
				
				if( !mRecorder.isRecording() ){
					DebugLog.LogD( "start audio recording." );
					ret = mRecorder.startRecording( mAudioReocrdListener );
				}else{
					DebugLog.LogD( "audio recorder is recording." );
					ret = ErrorCode.SUCCESS;
				}//end of if-else not recording
			}else{
				ret = ErrorCode.ERROR_AIMIC_CREATE_HANDLE;
			}//end of if-else recorder not null
		}//end of if valid
		
		return ret;
	}
	
	private void stopAudioRecord(){
		if( null != mRecorder ){
			DebugLog.LogD( "stop audio record" );
			mRecorder.stopRecording();
			mAudioReocrdListener.onStop();
			mRecorder = null;
		}
	}
	
	private class AudioWritingThread extends Thread{
		private final int MIN_SLEEP = 1;
		private final int MAX_SLEEP = 16;
		private final int MAX_TRY = 1000;
		private final LengthLimitedBuffer mSrcAudios = AIMicImpl.this.mSrcAudios;
		
		private int mPriority = Thread.MAX_PRIORITY;
		
		private final HashMap<Integer, Integer> mPriorityMap = new HashMap<Integer, Integer>();
		
		public AudioWritingThread( final String name ){
			super( name );
			
			mPriorityMap.put( AIMic.PRIORITY_MIN, Thread.MIN_PRIORITY );
			mPriorityMap.put( AIMic.PRIORITY_NORMAL, Thread.NORM_PRIORITY );
			mPriorityMap.put( AIMic.PRIORITY_MAX, Thread.MAX_PRIORITY );
		}
		
		private SpeechError mError = null;
		
		public SpeechError getError(){
			return mError;
		}
		
		public void resetError(){
			mError = null;
		}
		
		@Override
		public void run() {
			DebugLog.LogD( "aimic audio writing thread enter" );
			
			updateThreadPriority( mPriority );
			
			final int MIN_LOG_COUNT = 30;
			int buf_log_count = 0;
			
			int errorCode = 0;
			int sleep_time = 0;
			int try_count = 0;
			LengthLimitedBuffer.BufferEntry bufferEntry = null;
			while( isValid() ){
				try{
					bufferEntry = mSrcAudios.popData();
					if( null != bufferEntry ){
						++buf_log_count;
						if( MIN_LOG_COUNT <= buf_log_count )
						{
							buf_log_count = 0;
							DebugLog.LogD( "current buf container size in aimic thread is "+ mSrcAudios.getCurrnetBlockUsed() );
						}
						
						sleep_time = 0;
						try_count = 0;

						errorCode = AIMIC.AIMICAudioWrite( AIMIC.getHandler(), bufferEntry.getKey(), 0, bufferEntry.getValue() );
						
						mSrcAudios.recycle( bufferEntry );
						if( ErrorCode.SUCCESS != errorCode ){
							DebugLog.LogE( "AIMICAudioWrite error: "+errorCode );
							mAIMicListener.onError( new SpeechError(errorCode) );
						}
						
					}else{
						releaseAudioBuffers();
						if( MAX_TRY <= try_count ){
							try_count = 0;
							
							synchronized( this ){
								if( mSrcAudios.isEmpty() ){
									DebugLog.LogD( "aimic audio writing will suspend" );
									wait();
									DebugLog.LogD( "aimic audio writing is waked" );
								}	
							}//end of synchronized
							
						}else{
							++try_count;
							sleep_time = Math.min( sleep_time+MIN_SLEEP, MAX_SLEEP );
							sleep( sleep_time );
						}//end of match max try
					}//end of if-else buffer entry not null
				}catch( UnsatisfiedLinkError e ){
					DebugLog.LogE( e );
					mError = new SpeechError( ErrorCode.ERROR_UNSATISFIED_LINK );
				}catch( Throwable e ){
					DebugLog.LogE( e );
					mError = new SpeechError( ErrorCode.ERROR_UNKNOWN );
				}//end of try-catch
			}//end of while is valid
			
			mSrcAudios.clear();
			
			DebugLog.LogD( "aimic audio writing thread exited" );
		}//end of function run
		
		public void setThreadPriority( final int targetAIMicPriority ){
			final Integer val = this.mPriorityMap.get( targetAIMicPriority );
			mPriority = null!=val ? val : mPriority;
			DebugLog.LogD( "set priority, target aimic priority: "+targetAIMicPriority+", real target priority: "+mPriority );
			
			updateThreadPriority( mPriority );
		}
		
		private void updateThreadPriority( final int targetPriority ){
			DebugLog.LogD( "update priority enter, target priority: "+targetPriority+", current priority: "+this.getPriority() );
			
			if( targetPriority!= this.getPriority() ){
				try{
					LinkedList<ThreadGroup> groups = new LinkedList<ThreadGroup>();
					ThreadGroup threadGroup = this.getThreadGroup();
					while( null != threadGroup ){
						DebugLog.LogD( "thread group name: "+threadGroup.getName() );
						try{
							if( targetPriority<=threadGroup.getMaxPriority() ){
								DebugLog.LogD( "get a thread group has target priority" );
								break;
							}else{
								threadGroup.checkAccess();
								groups.addFirst( threadGroup );
								
								threadGroup = threadGroup.getParent();
							}
							
						}catch( Throwable e ){
							DebugLog.LogE( "exception while improve thread group priority" );
							DebugLog.LogE( e );
							break;
						}
					}//end of while
					
					for( ThreadGroup group : groups ){
						if( targetPriority > group.getMaxPriority() ){
							group.setMaxPriority( targetPriority );
						}
					}//end of for
					
					this.setPriority( targetPriority );
					DebugLog.LogD( "set thread priority to target, after that is: "+this.getPriority() );
				}catch( Throwable e ){
					DebugLog.LogE( "exception while set thread priority" );
					DebugLog.LogE( e );
				}//end of try-catch
			}//end of if-else need improve
			
			DebugLog.LogD( "update priority leave" );
		}
		
		private static final int MIN_RELEASE_PERIOD = 2 * 60 * 1000;	
		private long mFirstEmptyTime = System.currentTimeMillis();
		private boolean mIsFirstEmpty = true;
		private void releaseAudioBuffers(){
			if( BUF_MODE_STRICT!=AIMicImpl.this.mBufMode
					&& MIN_RELEASE_PERIOD>=System.currentTimeMillis()-mFirstEmptyTime ){
				final long avaibleLen = mSrcAudios.getAvailable();
				if( PER_BUF_LEN<avaibleLen
					&& avaibleLen+mSrcAudios.getCurrentLength()>BUF_LEN ){
					mIsFirstEmpty = ! mIsFirstEmpty;
					if( ! mIsFirstEmpty ){
						mSrcAudios.gc();
					}
				}
				
				mFirstEmptyTime = System.currentTimeMillis();
			}//end of if is time to check
		}//end of function releaseAudioBuffers
	}//end of class AudioWritingThread
	
	private static class AIMicListener implements Listener{
		private final HashSet<IvwDataListener> mIvwDataListeners = new HashSet<IvwDataListener>();
		private final HashSet<AsrDataListener> mAsrDataListeners = new HashSet<AsrDataListener>();
		private final Object mIvwSync = new Object();
		private final Object mAsrSync = new Object();
		
		private final int BYTES_OF_PER_MS = 32;	// Must be controlled by parameter later
		private final int DEF_ASR_BUF_TIME = 4000;
		private final int BUF_LEN = DEF_ASR_BUF_TIME * BYTES_OF_PER_MS;
		private final int PER_BUF_LEN = 512;
		private final LengthLimitedBuffer mAsrAudios = new LengthLimitedBuffer( BUF_LEN, PER_BUF_LEN, 0, false, true );
		
		private final int MSG_ID_WAKEUP = 1;
		
		private boolean mIsAsrAudioBuffering = true;
		private int mAsrBufferingLength = BUF_LEN;
		
		public boolean isEmpty(){
			return (null==mIvwDataListeners || mIvwDataListeners.isEmpty()) 
					&& (null==mAsrDataListeners || mAsrDataListeners.isEmpty() );
		}
		
		public void clear(){
			if( null != mIvwDataListeners ){
				mIvwDataListeners.clear();
			}
			
			if( null != mAsrDataListeners ){
				mAsrDataListeners.clear();
			}
		}
		
		/**
		 * 
		 * @param ms [0, 4000]
		 */
		public void setAsrBufferingTime( final int ms ){
			mAsrBufferingLength = ms * BYTES_OF_PER_MS;
		}
		
		public void registerListener( final IvwDataListener listener ){
			if( null != listener ){
				synchronized( mIvwSync ){
					mIvwDataListeners.add( listener );
				}//end of synchronized
			}//end of if listener not null
		}
		
		public void registerListener( final AsrDataListener listener ){
			if( null != listener ){
				synchronized( mAsrSync ){
					mAsrDataListeners.add( listener );
				}//end of synchronized
			}//end of if listener not null
		}
		
		public void unregisterListener( final IvwDataListener listener ){
			if( null != listener ){
				synchronized( mIvwSync ){
					mIvwDataListeners.remove( listener );
				}//end of synchronized
			}//end of if listener not null
		}
		
		public void unregisterListener( final AsrDataListener listener ){
			if( null != listener ){
				synchronized( mAsrSync ){
					mAsrDataListeners.remove( listener );
				}//end of synchronized
			}//end of if listener not null
		}
		
		@Override
		public void onWakeupAudio(byte[] audioData, int dataLen, int arg1, Object arg2) {
			try{
				//DebugLog.LogS( "onWakeupAudio enter "+ System.currentTimeMillis() );
				synchronized( mIvwSync ){
					//DebugLog.LogS( "onWakeupAudio sync in" );
					for( Iterator<IvwDataListener> iterator=mIvwDataListeners.iterator(); iterator.hasNext();  ){
						//DebugLog.LogS( "onWakeupAudio for in" );
						final IvwDataListener listener = iterator.next();
						//DebugLog.LogS( "onWakeupAudio get listener" );
						listener.onWakeupAudio( audioData, dataLen, arg1, arg2 );
					}
				}//end of synchronized
				//DebugLog.LogS( "onWakeupAudio leave "+ System.currentTimeMillis() );
			}catch( Throwable e ){
				//DebugLog.LogE( e );
				onError( new SpeechError(e, ErrorCode.ERROR_UNKNOWN) );
			}//end of try-catch
			
		}

		@Override
		public void onWakeupMsg(int msg, int arg1, int arg2, byte[] sid, int sidLen, byte[] result, int resultLen, byte[] audio, int audioLen ) {
			try{
				DebugLog.LogS( "onWakeupMsg enter" );
				if( MSG_ID_WAKEUP == msg ){
					synchronized( mAsrSync ){
						mIsAsrAudioBuffering = 0<mAsrBufferingLength;
					}
				}//end of is wakeup msg
				
				synchronized( mIvwSync ){
					for( Iterator<IvwDataListener> iterator=mIvwDataListeners.iterator(); iterator.hasNext();  ){
						final IvwDataListener listener = iterator.next();
						listener.onWakeupMsg( msg, arg1, arg2, sid, sidLen, result, resultLen, audio, audioLen );
					}
				}//end of synchronized
				DebugLog.LogS( "onWakeupMsg leave" );
			}catch( Throwable e ){
				DebugLog.LogE( e );
				onError( new SpeechError(e, ErrorCode.ERROR_UNKNOWN) );
			}//end of try-catch
		}
		
		@Override
		public void onRecogAudio(byte[] audioData, int dataLen, int arg1, Object arg2) {
			try{
				//DebugLog.LogS( "onRecogAudio enter "+ System.currentTimeMillis() );
				synchronized( mAsrSync ){
					mIsAsrAudioBuffering = mIsAsrAudioBuffering && mAsrDataListeners.isEmpty();
					if( mIsAsrAudioBuffering ){
						final long curLen = mAsrAudios.getCurrentLength();
						if( curLen+dataLen > mAsrBufferingLength ){
							DebugLog.LogD( curLen+" matched max buffering len: "+mAsrBufferingLength+", will be clean" );
							mIsAsrAudioBuffering = false;
							mAsrAudios.clear();
						}else{
							mAsrAudios.put( audioData, 0, dataLen );
						}
					}else{
						for( Iterator<AsrDataListener> iterator=mAsrDataListeners.iterator(); iterator.hasNext();  ){
							final AsrDataListener listener = iterator.next();
							
							if( !mAsrAudios.isEmpty() ){
								LengthLimitedBuffer.BufferEntry bufEntry = null;
								for( bufEntry=mAsrAudios.popData(); null != bufEntry; bufEntry=mAsrAudios.popData() ){
									listener.onRecogAudio( bufEntry.getKey(), bufEntry.getValue(), 0, null );
									mAsrAudios.recycle( bufEntry );
								}//end of for all buffers saved
								
								mAsrAudios.clear();
							}//end of buffer not empty
							
							listener.onRecogAudio( audioData, dataLen, arg1, arg2 );
						}//end of for all listeners
					}//end of if buffering
				}//end of synchronized
				
				//DebugLog.LogS( "onRecogAudio leave "+ System.currentTimeMillis() );
			}catch( Throwable e ){
				DebugLog.LogE( e );
				onError( new SpeechError(e, ErrorCode.ERROR_UNKNOWN) );
			}//end of try-catch
		}

		public void onError( final SpeechError error ){
			DebugLog.LogE( error );
			synchronized( mIvwSync ){
				for( Iterator<IvwDataListener> iterator=mIvwDataListeners.iterator(); iterator.hasNext();  ){
					final IvwDataListener listener = iterator.next();
					listener.onError( error.getErrorCode() );
				}
			}//end of synchronized
			
			synchronized( mAsrSync ){
				for( Iterator<AsrDataListener> iterator=mAsrDataListeners.iterator(); iterator.hasNext();  ){
					final AsrDataListener listener = iterator.next();
					listener.onError( error.getErrorCode() );
				}
			}//end of synchronized
		}
	}//end ofclass AIMicListener

	private class AudioRecordListener implements AudioRecorder.RecorderListener{
		private int mErrorCode =ErrorCode.SUCCESS;
		
		@Override
		public void onAudioData(byte[] audio, int dataLen ) {
			//DebugLog.LogS( "audio record onAudioData enter, "+ System.currentTimeMillis() );
			if( isSaveAlsaData() ){
				saveAlsaData( audio, dataLen );
			}
			
			mErrorCode = writeAudio( audio, 0, dataLen );
			if( ErrorCode.SUCCESS != mErrorCode ){
				mAIMicListener.onError( new SpeechError(mErrorCode) );
			}
			//DebugLog.LogS( "audio record onAudioData leave, "+ System.currentTimeMillis() );
		}//end of function onPcmData
		
		private final int ALSA_PER_BUF_LEN = 512 * 96;
		private final int ALSA_BUF_LEN = 512 * 1000 * 1;
		private final int ALSA_FILE_LEN = ALSA_BUF_LEN * 60 * 2;
		private final LengthLimitedBuffer mSrcBuffer_1 = new LengthLimitedBuffer( ALSA_BUF_LEN, ALSA_PER_BUF_LEN, 0, true, false );
		private final LengthLimitedBuffer mSrcBuffer_2 = new LengthLimitedBuffer( ALSA_BUF_LEN, ALSA_PER_BUF_LEN, 0, true, false );
		private volatile LengthLimitedBuffer mCurUsingBuffer = mSrcBuffer_1;
		private volatile LengthLimitedBuffer mCurFlushBuffer = mSrcBuffer_2;
		private final String mAlsaPcmPath = Environment.getExternalStorageDirectory().getPath()+"/aimic_alsa.pcm";
		
		//private final Object mSyncAlsa = new Object();
		private Thread mAlsaSaveThread = null;
		private boolean mIsSaveAlsaData = false;
		private boolean mIsSaving = false;
		
		public void onStop(){
			mIsSaving = false;
			if( isSaveAlsaData() ){
				flushAlsaAudio( 0, true );
			}
		}
		
		public void setSaveAlsaData( final boolean isSaved ){
			mIsSaveAlsaData = isSaved;
		}
		
		private boolean isSaveAlsaData(){
			return mIsSaveAlsaData;
		}
		
		private void saveAlsaData( final byte[] audio, final int dataLen ){
			if( null == mAlsaSaveThread ){
				File file = new File( mAlsaPcmPath );
				if( file.exists() ){
					file.delete();
				}
				
				mSrcBuffer_1.clear();
				mSrcBuffer_2.clear();
				mCurUsingBuffer = mSrcBuffer_1;
				mCurFlushBuffer = mSrcBuffer_2;
				mIsSaving = true;
				startSaveAlsaAudio();
			}//end of mAlsaSaveThread is null
			
			try{
				flushAlsaAudio( dataLen, false );
				
				synchronized( mCurUsingBuffer ){
					mCurUsingBuffer.put( audio, 0, dataLen );
				}//end of synchronized
			}catch( Throwable e ){
				DebugLog.LogE( e );
				mAIMicListener.onError( new SpeechError(e, ErrorCode.ERROR_UNKNOWN) );
			}
		}
		
		private boolean saveFileFromMemory( final RandomAccessFile dstFile, final MemoryFile srcMemFile, final int srcOffset, final int length ){
			boolean ret = true;
			
			try{
				if( null!=dstFile && null!=srcMemFile && 0<=srcOffset && 0<=length ){
					final int srcEndOffset = srcOffset + length;
					final byte[] buffer = new byte[ 64 * 1024 ];
					int dataLen = 0;
					int offset = srcOffset;
					while( srcEndOffset > offset ){
						dataLen = Math.min( buffer.length, srcEndOffset-offset );
						if( srcMemFile.readBytes( buffer, offset, 0, dataLen ) != dataLen ){
							DebugLog.LogE( "saveFileFromMemory error: read bytes length error!" );
							ret = false;
							break;
						}
						offset += dataLen;
						dstFile.write( buffer, 0, dataLen );
					}//end of while
				}else{
					ret = false;
					DebugLog.LogE( "saveFileFromMemory error: arguments error" );
				}//end of if-else offset > 0
			}catch( Throwable e ){
				ret = false;
				DebugLog.LogE( e );
			}//end of try-catch
			
			return ret;
		}
		
		private void startSaveAlsaAudio(){
			mAlsaSaveThread = new Thread( "AlsaAudioSavingThread" ){
				@Override
				public void run(){
					DebugLog.LogD( "alsa data save thread enter" );
					
					SpeechError error = null;
					MemoryFile memFile = null;
					final int BUF_FILE_LEN = ALSA_FILE_LEN+ALSA_BUF_LEN;
					int curOffset = 0;
					boolean isCycle = false;
					RandomAccessFile fileAccessor = null;
					
					try{
						memFile = new MemoryFile( null, BUF_FILE_LEN );
						memFile.allowPurging( false );
						
						LengthLimitedBuffer.BufferEntry bufEntry = null;
						
						while( mIsSaving ){
							synchronized( mCurFlushBuffer ){
								while( !mCurFlushBuffer.isEmpty() ){
									bufEntry = mCurFlushBuffer.popData();
									memFile.writeBytes( bufEntry.getKey(), 0, curOffset, bufEntry.getValue() );
									curOffset += bufEntry.getValue();
									mCurFlushBuffer.recycle( bufEntry );
								}
									
								mCurFlushBuffer.clear();
							}//end of synchronized
							
							if( ALSA_FILE_LEN <= curOffset ){
								//DebugLog.LogS( "alsa data save thread file will be truncated" );
								isCycle = true;
								curOffset = 0;
								//DebugLog.LogS( "alsa data save thread file is truncated" );
							}
							
							synchronized( this ){
								if( mCurFlushBuffer.isEmpty() ){
									//DebugLog.LogS( "alsa data save thread waiting" );
									this.wait();
									//DebugLog.LogS( "alsa data save thread waked" );
								}else{
									//DebugLog.LogS( "alsa data save thread buffer has been updated." );
								}
							}//end of synchronized
						}//end of while is recording
						
						synchronized( mCurFlushBuffer ){
							while( !mCurFlushBuffer.isEmpty() ){
								bufEntry = mCurFlushBuffer.popData();
								memFile.writeBytes( bufEntry.getKey(), 0, curOffset, bufEntry.getValue() );
								curOffset += bufEntry.getValue();
								mCurFlushBuffer.recycle( bufEntry );
							}
								
							mCurFlushBuffer.clear();
						}//end of synchronized
						
						if( null == error ){
							fileAccessor = new RandomAccessFile( mAlsaPcmPath, "rw" );
							
							final int lastLen = isCycle ? ALSA_FILE_LEN-curOffset : 0;
							if( 0<lastLen 
									&& !saveFileFromMemory(fileAccessor, memFile, curOffset, lastLen) ){
								error = new SpeechError( ErrorCode.ERROR_UNKNOWN );
								DebugLog.LogE( "save last memory file data failed!" );
							}//end of if last file len need > 0
							
							if( null == error
									&& !saveFileFromMemory(fileAccessor, memFile, 0, Math.min(ALSA_FILE_LEN, curOffset)) ){
								error = new SpeechError( ErrorCode.ERROR_UNKNOWN );
								DebugLog.LogE( "save current memory file data failed!" );
							}
						}//end of error is null
						
						
					}catch( Throwable e ){
						DebugLog.LogE( e );
						DebugLog.LogE( "保存文件失败！" );
						error =new SpeechError( e, ErrorCode.ERROR_UNKNOWN );
					}//end of try-catch
					
					try{
						if( null != memFile ){
							memFile.close();
						}
						
						if( null != fileAccessor ){
							fileAccessor.close();
						}
					}catch( Throwable e ){
						DebugLog.LogE( e );
						DebugLog.LogE( "关闭文件异常！" );
					}//end of try-catch
					 
					mAlsaSaveThread = null;
					
					if( null != error ){
						mAIMicListener.onError( error );
					}
					
					DebugLog.LogD( "alsa data save thread exit" );
				}//end of function run
			};
			
			mAlsaSaveThread.start();
		}
		
		private void flushAlsaAudio( final int bufLen, final boolean force ){
			LengthLimitedBuffer flushBuffer = null;
			
			synchronized( mCurUsingBuffer ){
				if( force || mCurUsingBuffer.getCurrentLength()+bufLen >= ALSA_BUF_LEN ){
					flushBuffer = mCurUsingBuffer;
					mCurUsingBuffer = flushBuffer.equals(mSrcBuffer_2) ? mSrcBuffer_1 : mSrcBuffer_2;
				}
			}//end of synchronized current
			
			if( null != flushBuffer ){
				synchronized( mCurFlushBuffer ){
					mCurFlushBuffer = flushBuffer;
				}
				
				if( null != mAlsaSaveThread ){
					synchronized( mAlsaSaveThread ){
						if( Thread.State.WAITING == mAlsaSaveThread.getState() ){
							mAlsaSaveThread.notify();
						}
					}
				}//end of thread not null
				
				synchronized( mCurUsingBuffer ){
					if( 0 != mCurUsingBuffer.getCurrentLength() ){
						DebugLog.LogE( "Error: buffer is not null when exchanged!" );
						mAIMicListener.onError( new SpeechError(ErrorCode.ERROR_UNKNOWN) );
					}//end of buffer check
				}//end of synchronized
			}//end of if exchange buffer
		}
	}//end of class AlsaListener
}