package com.iflytek.cloud.msc.mfv;


import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.HashMap;

import org.json.JSONException;
import org.json.JSONObject;

import android.content.Context;
import android.os.Bundle;
import android.os.HandlerThread;
import android.os.Message;
import android.os.SystemClock;
import android.text.TextUtils;

import com.iflytek.cloud.ErrorCode;
import com.iflytek.cloud.IdentityListener;
import com.iflytek.cloud.IdentityResult;
import com.iflytek.cloud.SpeechConstant;
import com.iflytek.cloud.SpeechError;
import com.iflytek.cloud.SpeechEvent;
import com.iflytek.cloud.msc.module.MscHandler;
import com.iflytek.cloud.msc.module.SessPerflog;
import com.iflytek.cloud.msc.util.DataLogger;
import com.iflytek.cloud.msc.util.DataUtil;
import com.iflytek.cloud.msc.util.NetworkUtil;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.msc.util.log.PerfLogger;
import com.iflytek.cloud.param.HashParam;
import com.iflytek.cloud.param.MscKeys;
import com.iflytek.msc.MSC;

/**
 * 身份验证具体功能类
 * 
 * @author iflytek
 */
public class MfvVerifier extends MscHandler {
	private final static String TAG = MfvVerifier.class.getSimpleName();
	private final static String KEY_SUB = "sub";
	private final static String KEY_SSUB = "ssub";
	private final static String KEY_RET = "ret";
	private final static String KEY_SST = "sst";
	private final static String KEY_MVER = "mver";
	private final static String KEY_SRET = "sret";
//	private final static String KEY_RESULT ="result";
	private final static String KEY_SCENES ="scenes";
	
	private boolean mVadEnable = true;
	
	// 外部回调接口
	protected volatile IdentityListener mListener = null;
	// 是否是写第一块数据
	protected boolean mFirstDataWrite = false;
	// 是否获取到第一个结果
	protected boolean mFirstResultArrive = false;
	// 是否用户主动停止写入
	protected boolean mUserStop = false;

	protected MfvSession mSession = new MfvSession();

	// 会话产生的上下行流量
	public static int mUpflow = 0;
	public static int mDownflow = 0;

	private DataLogger mDatalogs;
	// 验证结果
	protected String mResult = null;
	
	// 性能结构化日志
	protected SessPerflog mPerflog = new SessPerflog();
	// 用来保存每个子业务的参数
	private HashMap<String, HashParam> mSSubParams = null;
	// 用来保存每个子业务的参数
	private HashMap<String, StringBuffer> mSSubParamsString = null;
	
	// 子业务调writeData写数据标记，用于设置data_status，true则为第一块数据
	private HashMap<String, Boolean> mSSubFirstDataBlock;

	private String mSessionID = null;
	
	public MfvVerifier(Context context, HashParam param, HandlerThread mHandlerThread) {
		super(context, mHandlerThread);
		mDatalogs = new DataLogger();
		mUserStop = false;
		mSSubParams = new HashMap<String, HashParam>();
		mSSubParamsString = new HashMap<String, StringBuffer>();
		mSSubFirstDataBlock = new HashMap<String, Boolean>();
		setParams(param);
	}

	/**
	 * 获取数据缓存队列。
	 */
	public DataLogger getBuffer() {
		return mDatalogs;
	}
	
	/**
	 * 通知音量变化。
	 * @param data
	 * @param volume 音量，0-30。
	 */
	public void callbackVolume(byte[] data, int volume) {
		if (null != mListener && isRunning()) {
			mListener.onEvent(SpeechEvent.EVENT_VOLUME, volume, 0, null);
		}
	}

	/**
	 * 开始会话。
	 * @param listener 身份验证监听器
	 */
	public synchronized void startWorking(IdentityListener listener) {
		mListener = listener; 
		DebugLog.LogD("startWorking called");
		// 启动会话
		start();
	}

	@Override
	protected void start() {
		mPerflog.startSession(getParam());
		super.start();
	}

	@Override
	protected void onMsgProcess(Message msg) throws Throwable, SpeechError {
		super.onMsgProcess(msg);

		switch (msg.what) {
		case MSG_START:
			proc_Msg_Start();
			break;
		case MSG_SESSION_BEGIN:
			proc_Msg_Session_Begin();
			break;
		case MSG_SPEECH_TIMEOUT:
			vadEndCall();
			break;
		case MSG_RECORD_AUDIO:
			proc_Msg_Record_Data(msg);
			break;
		case MSG_RECORD_STOPD:
			proc_Msg_Record_Stoped(msg);
			break;
		case MSG_RESULT:
			proc_Msg_Result(msg);
			break;
		case MSG_NET_PERF:
			proc_Msg_Netperf();
			break;
		default:
			break;
		}
	}

	/**
	 * 启动消息处理
	 * @throws Exception
	 */
	protected void proc_Msg_Start() throws Exception {
		DebugLog.LogD("[mfv]start connecting");
		// 插入到消息头部，防止音频被预先处理
		sendMsg(MSG_SESSION_BEGIN, Priority.max, false, 0);
	}

	private int mSessionRetryCount = 0;
	
	/**
	 * MSC SessionBegin消息处理
	 * 特别注意点：缓冲音频的写入，必须保证在SessionBegin之后
	 * @throws Exception
	 */
	protected void proc_Msg_Session_Begin() throws Exception {
		// 网络检查
		boolean net_check = getParam().getBoolean(SpeechConstant.NET_CHECK, true);
		if (net_check) {
			NetworkUtil.checkNetwork(mContext);
		}

		PerfLogger.appendInfo(PerfLogger.SDK_SESSION_BIGNE, null);
		mPerflog.pushSessionInfo(SessPerflog.KEY_APP_SSB);
		int code = mSession.sessionBegin(mContext, null, this);
		
		if (code == 0 && null != mSession.mClientID) {
			// 注册回调接口
			if (isRunning()) {
				MSC.QMFVRegisterNotify(mSession.mClientID, "rsltCb", "stusCb", "errCb", this);
				setStatus(Status.recording);
				
				if (getParam().getBoolean(SpeechConstant.ASR_NET_PERF, false)) {
					sendMsg(MSG_NET_PERF, Priority.max, false, 0);
				}
			}
		} else {
			// 延迟消息无法发送，必须通过sleep方式，否则会造成音频丢失或处理错误
			// 需要优先级放到最高，否则会造成录音消息被预先处理，造成报错
			mSessionRetryCount++;
			if (mSessionRetryCount > 40)
				throw new SpeechError(code);
			else if (isRunning()) {
				Thread.sleep(15);
				sendMsg(MSG_SESSION_BEGIN, Priority.max, false, 0);
			}
		}
	}

	/**
	 * 写数据消息处理
	 * @param msg
	 * @throws Exception
	 */
	protected void proc_Msg_Record_Data(Message msg) throws Exception {
		HashMap<String, Object> map = (HashMap<String, Object>) msg.obj;
		
		byte[] data = (byte[]) map.get("data");
		
		String ssub = (String) map.get(KEY_SSUB);
		HashParam hParam = mSSubParams.get(ssub);
		StringBuffer param = mSSubParamsString.get(ssub);
		String path = hParam.getString(SpeechConstant.MFV_DATA_PATH);
		if (!TextUtils.isEmpty(path) && null != data) {
			mDatalogs.addData(path, data);
		}
		
		Boolean firstDataBlock = mSSubFirstDataBlock.get(ssub);
		if (null == firstDataBlock) {
			firstDataBlock = true;
		}
		
		if ("ivp".equals(ssub) && mVadEnable) {
			// 只有ivp子业务且打加开vad时，才有音量通知
			uploadData(param, data, firstDataBlock, true);
		} else {
			uploadData(param, data, firstDataBlock, false);
		}
		
		if (firstDataBlock) {
			// 写完第一块数据后，标志置为false
			mSSubFirstDataBlock.put(ssub, false);
		}
	}
	
	/**
	 * 停止写入数据。
	 * @param ssub 子业务类型
	 * @param userStop 是否用户主动停止
	 * @return 是否成功
	 */
	public synchronized boolean stopWrite(String ssub, boolean userStop) {
		DebugLog.LogD("stopRecognize, current status is :" + getStatus()
				+ " usercancel : " + userStop);
		mPerflog.pushSessionInfo(SessPerflog.KEY_APP_STOP);
		
		// 用户主动点击录音停止
		mUserStop = userStop;
		sendMsg(obtainMessage(MSG_RECORD_STOPD, ssub));
		return true;
	}

	/**
	 * 数据停止写入消息处理
	 * @throws SpeechError
	 * @throws IOException
	 * @throws InterruptedException
	 */
	private void proc_Msg_Record_Stoped(Message msg) throws SpeechError, IOException, InterruptedException{	
		DebugLog.LogD( "recording stop" );
		mPerflog.pushSessionInfo(SessPerflog.KEY_APP_LAU);
		
		String ssub = (String) msg.obj;
		// 子业务数据写完，重置子业务的写第一块数据标记为true
		mSSubFirstDataBlock.put(ssub, true);
		// upload end flag
		mSession.pushEndFlag((String)msg.obj);
		// 最低优先级消息，会被自动清除
		updateTimeoutMsg();
	}
	
	/**
	 * 取消验证
	 * 
	 * @notifyError 是否有错误通知
	 */
	@Override
	public void cancel(boolean notifyError) {
		if(notifyError && isRunning() && mListener != null){
			DebugLog.LogD("cancel");
			mListener.onError(new SpeechError(ErrorCode.ERROR_INTERRUPT));
		}
		// 用户主动点击取消
		if(getStatus() == Status.recording){
			mUserStop = true;
		}
		super.cancel(notifyError);
	}

	/**
	 * 返回外部当前msc的网络连接状态值，lua版本中可见
	 * 100ms返回一次数据
	 */
	public void proc_Msg_Netperf()
	{
		// 100ms返回一次
		if(isRunning())
		{
			int netpref = mSession.getIntValue(TAG_NETPERF);
			if(mListener != null)
				mListener.onEvent(SpeechEvent.EVENT_NETPREF, netpref, 0,null);
			// 继续开启下一次通知
			sendMsg(MSG_NET_PERF, Priority.normal,false,100);
		}
	}

	@Override
	protected void onEnd(SpeechError error) {
		DebugLog.LogD("onSessionEnd");
		mUpflow = mSession.getIntValue(TAG_UPFLOW);
		mDownflow = mSession.getIntValue(TAG_DOWNFLOW);
		getSessionID();
		
		if(null == mResult && error == null 
				&& getParam().getBoolean(SpeechConstant.ASR_NOMATCH_ERROR, true))
		{
			error = new SpeechError(ErrorCode.MSP_ERROR_NO_DATA);
		}
		
		//结构化日志增加应用层错误码，成功默认返回0
		if(null != error)
			mPerflog.pushSessionInfo( SessPerflog.KEY_APP_RET, error.getErrorCode(), false );
		else
			mPerflog.pushSessionInfo( SessPerflog.KEY_APP_RET, 0, false );

		// 写入录音启动时间、用户是否主动停止等消息

		mPerflog.pushSessionInfo(SessPerflog.KEY_REC_USTOP, mUserStop ? "1" : "0",false);
		mSession.setParam(SessPerflog.KEY_SESSIONINFO, mPerflog.getSessionInfo());

		PerfLogger.appendInfo(PerfLogger.SESSION_END_BEGIN, null);
		if(mUserCancel)
			mSession.sessionEnd("user abort");
		else if(error != null)
			mSession.sessionEnd("error" + error.getErrorCode());
		else
			mSession.sessionEnd("success");
		PerfLogger.appendInfo(PerfLogger.SESSION_END_END, null);
		
		super.onEnd(error);

		if (mListener != null) {
			if (mUserCancel) {
				// mListener.onCancel();
			} else {
				// auto exit
				if (error != null)
					mListener.onError(error);
			}
			mListener.onEvent(SpeechEvent.EVENT_SESSION_END, 0, 0, null);
		}
		mListener = null;
	}

	/**
	 * called from VAD check
	 */
	public void vadEndCall() {
		if (Status.recording == getStatus()) {
			DebugLog.LogD( "mfv msc vadEndCall" );
			// 通过onEvent接口抛出VAD事件
			if (mListener != null) {
				mListener.onEvent(SpeechEvent.EVENT_VAD_EOS, 0, 0, null);
			}
			
			stopWrite("ivp", false);
		}
	}

	/**
	 * 通知外部业务结果。
	 * @param isLast
	 * @throws SpeechError
	 * @throws UnsupportedEncodingException
	 */
	private void notifyResult(boolean isLast, byte[] resultByte) throws SpeechError, UnsupportedEncodingException{
		String result = null;
		mStatusBegin = SystemClock.elapsedRealtime();
		
		if (resultByte != null && resultByte.length > 0) {
			result = new String(resultByte, DataUtil.UTF8);
			// 将json结果中的“return”替换成“ret”，统一格式
			result = result.replace("\"return\"", "\"ret\"");
		} else {
			throw new SpeechError(ErrorCode.MSP_ERROR_NO_DATA);
		}

		mResult = result;
		if (null != mListener && isRunning()) {
			// 需要放在结果回调前面，确保回调最后结果时已经拿到sessionID
			// 通过onEvent抛出服务端会话id
			Bundle bundle = new Bundle();
			bundle.putString(SpeechEvent.KEY_EVENT_SESSION_ID, getSessionID());
			mListener.onEvent(SpeechEvent.EVENT_SESSION_ID, 0, 0, bundle);

			// 原始结果
			IdentityResult reresult = new IdentityResult(result);
			// 日志
			PerfLogger.appendInfo(PerfLogger.GET_RESULT, null);
			mListener.onResult(reresult, isLast);
		}

		DebugLog.LogD("msc result time:" + System.currentTimeMillis());

		if (isLast) {
			exit(null);
		}
	}

	@Override
	protected void onParseParam() {
		// 语音最长输入时间
		mSpeechTimeOut = getParam().getInt(SpeechConstant.KEY_SPEECH_TIMEOUT, mSpeechTimeOut);
		DebugLog.LogD("mSpeechTimeOut=" + mSpeechTimeOut);
		super.onParseParam();
	}
	

	/**
	 * 状态回调接口
	 * 
	 * @param clientSessionID 会话id
	 * @param type 类型
	 * @param status 状态
	 * @param param1 回调参数
	 * @param param2 回调参数
	 */
	void stusCb(char[] clientSessionID, int type, int status, byte[] param1, int param2) {
		DebugLog.LogD(TAG, "stusCb:" + status + ",type:" + type);
		if (0 == type && 3 == status) {
			// VAD后端点消息
			vadEndCall();
		}
		
		// 子业务出错
		if (1 == type) {
			String scenes = getParam().getString(KEY_SCENES, "");
			String[] ssubs = scenes.split("\\|");
			
			if (null != ssubs) {
				if (ssubs.length < 2) {
					// 单业务场景，不通过onResult抛出sret
					return;
				}
			}
			
			// status为sret，param2为ssub
			try {
				String ssub = new String(param1, DataUtil.UTF8);
				int sret = status;
				
				JSONObject json = new JSONObject();
				json.put(KEY_SUB, ssub);
				json.put(KEY_SRET, sret);
				json.put(KEY_RET, sret);
				
				HashParam ssubParam = mSSubParams.get(ssub);
				String sst = ssubParam.getString(KEY_SST);
				
				json.put(KEY_SST, sst);
				
				if (null != mListener) {
					IdentityResult result = new IdentityResult(json.toString());
					mListener.onResult(result, true);
				}
			} catch (UnsupportedEncodingException e) {
				DebugLog.LogE(e);
			} catch (JSONException e) {
				DebugLog.LogE(e);
			}
		}
	}
	/**
	 * 结果回调接口
	 * 
	 * @param clientSessionID
	 * @param result
	 * @param len
	 * @param resultstatus
	 */
	void rsltCb(char[] clientSessionID, byte[] result, int len, int resultstatus) {
//		String sst = getParam().getString(KEY_SST);
//		if ("query".equals(sst) || "delete".equals(sst)) {
//			// 拼凑模型操作的json结果
//			JSONObject json = new JSONObject();
//			// 模型操作时，只可能是单一业务（ivp或ifr），mSSubParams里只有一个key，就是ssub
//			String ssub = (String) mSSubParams.keySet().toArray()[0];
//			HashParam params = mSSubParams.get(ssub);
//			String mver = params.getString(KEY_MVER, "");
//			
//			if (!"2.0".equals(mver)) {
//				if (TextUtils.isEmpty(mver)) {
//					try {
//						json.put(KEY_RET, ErrorCode.SUCCESS);
//						json.put(KEY_SUB, ssub);
//						json.put(KEY_SST, sst);
//						
//						if (null == result) {
//							json.put(KEY_RESULT, "true");
//						} else {
//							try {
//								json.put(KEY_RESULT, new JSONObject(new String(result)));
//							} catch (JSONException e) {
//								json.put(KEY_RESULT, new String(result));
//							}
//						}
//					} catch (JSONException e) {
//						DebugLog.LogE(e);
//					}
//					
//					result = json.toString().getBytes();
//				}
//			}
//		}
		
		// 识别结果回调消息，优先级最高
		if (result != null) {
			DebugLog.LogD(TAG, "rsltCb:" + resultstatus + "result:"
					+ new String(result));
		} else {
			DebugLog.LogI(TAG, "rsltCb:" + resultstatus
					+ "result:null");
		}
		Message msg = obtainMessage(MSG_RESULT, resultstatus, 0, result);
		// 消息队列在执行很耗时的任务 后至的结果数据被插入队列前，导致之前的结果无法展示
		// ps:概率很小
		if (hasMessages(MSG_RESULT)) {
			sendMsg(msg, Priority.normal, false, 0);
		} else {
			sendMsg(msg, Priority.max, false, 0);
		}
	}

	/**
	 * 出错回调接口。
	 * @param clientSessionID
	 * @param errorcode
	 * @param detailDesc
	 */
	void errCb(char[] clientSessionID, int errorcode, byte[] detailDesc) {
		DebugLog.LogD(TAG, "clientSessionID:"+ new String(clientSessionID) + "errorcode:" + errorcode);
		
		// 通过onEvent抛出服务端会话id
		Bundle bundle = new Bundle();
		bundle.putString(SpeechEvent.KEY_EVENT_SESSION_ID, getSessionID());
		mListener.onEvent(SpeechEvent.EVENT_SESSION_ID, 0, 0, bundle);
		
		exit(new SpeechError(errorcode));
	}

	/**
	 * 结果通知消息处理
	 * 	status:   2----unknown 0----has result 5----is last result
	 * @throws SpeechError
	 * @throws InterruptedException
	 * @throws UnsupportedEncodingException
	 */
	void proc_Msg_Result(Message msg) throws SpeechError, InterruptedException, UnsupportedEncodingException{
		int status = msg.arg1;
		byte[] data = (byte[]) msg.obj;
		//long start = SystemClock.elapsedRealtime();
		//		long t=SystemClock.elapsedRealtime();
		switch (status) {
		case 2://回调版本msc不返回2
			break;
		case 0:
			if (!mFirstResultArrive) {
				mFirstResultArrive = true;
				mPerflog.pushSessionInfo(SessPerflog.KEY_APP_FRS);
			}
			notifyResult(false, data);
			break;
		case 5:
			if (!mFirstResultArrive) {
				mFirstResultArrive = true;
				mPerflog.pushSessionInfo(SessPerflog.KEY_APP_FRS);
			}
			mPerflog.pushSessionInfo(SessPerflog.KEY_APP_LRS);
			notifyResult(true, data);
			break;
		default:
			break;
		}
		//DebugLog.LogD( "get result time is"+(SystemClock.elapsedRealtime()-start) );
	}

	/**
	 * 上传业务数据。
	 * @param data 			业务数据
	 * @param needVolNotify 是否需要音量通知
	 * @throws SpeechError
	 */
	protected void uploadData(StringBuffer params, byte[] data, boolean firstBlock, boolean needVolNotify) 
			throws SpeechError{	
		int length = (null == data) ? 0 : data.length;
		mSession.pushAudioData(params, data, length, firstBlock);
		
		if (needVolNotify) {
			int volume = mSession.getAudioVolume();
			DebugLog.LogI("QISRAudioWrite volume:" + volume);
			callbackVolume(data, volume);
		}
	}

	long time = 0L;
	public void writeData(String ssub, String params, byte[] data, int offset, 
			int length ) {
		// 处理参数
		if(!mSSubParams.containsKey(ssub)) {
			HashParam hp = new HashParam();
			hp.initWithParam(params);
			// 将session begin中的sst参数加入到业务参数中
			final String sst = getParam().getString(KEY_SST);
			hp.putParam(KEY_SST, sst, false);
			// 为ssub加入版本信息mver
			hp.putParam(KEY_MVER, "2.0", false);
			hp.putParam(KEY_SSUB, ssub);
			mVadEnable = hp.getBoolean(SpeechConstant.VAD_ENABLE,true);
			mSSubParams.put(ssub, hp);
			//保存一份String的直接传递到底层，避免多次调用导致CPU过高
			mSSubParamsString.put(ssub,new StringBuffer(hp.toString()));
		}
		if (isRunning()) {
			if(false == mFirstDataWrite){
				mFirstDataWrite = true;
				mPerflog.pushSessionInfo(SessPerflog.KEY_REC_START);
			}
			
			HashMap<String, Object> map = new HashMap<String, Object>();
			map.put(KEY_SSUB, ssub);
			byte[] temp = null;
			
			if (null != data) {
				// 拷贝内存，发送数据消息
				temp = new byte[length];
				System.arraycopy(data, offset, temp, 0, length);
			}
			map.put("data", temp);

			sendMsg(obtainMessage(MSG_RECORD_AUDIO, map));
		}
	}

	/**
	 * 返回结构化日志对象，便于外部写入。
	 * @return
	 */
	public SessPerflog getPerfLog() {
		return mPerflog;
	}

	/**
	 * 返回服务端ID。
	 */
	@Override
	public String getSessionID() {
		if( TextUtils.isEmpty(mSessionID) ){
			mSessionID = mSession.getSessionID();
		}
		return mSessionID;
	}
	
	/**
	 * 返回客户端ID。
	 */
	@Override
	public String getClientID() {
		return mSession.getClientID();
	}

}