package com.iflytek.cloud.msc.module;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.os.SystemClock;
import android.text.TextUtils;

import com.iflytek.cloud.msc.util.AppInfoUtil;
import com.iflytek.cloud.msc.util.DataUtil;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.param.HashParam;
import com.iflytek.cloud.param.MscKeys;

/**
 * 性能结构化日志对象，用于保存性能日志，处理生成json格式的结构化日志对象
 * @author yjzhao
 *
 */
public class SessPerflog {

	
	//性能信息
	JSONObject mSessionInfo = new JSONObject();
	
	//启动识别的相对时间，用来计算后续的时间偏移值
	long app_RelateStart = 0;
	//启动识别的绝对时间
	long app_AbsoStart = 0;
	
	
	/**
	 * 开始会话
	 * @author yjzhao
	 */
	public void startSession(HashParam param)
	{
		app_AbsoStart = System.currentTimeMillis();
		app_RelateStart = SystemClock.elapsedRealtime();
		pushSessionInfo(SessPerflog.KEY_APP_START, DataUtil.getComplexDateTime(app_AbsoStart), false);
		String callerappid = param.getString(MscKeys.KEY_CALLER_APPID);
		if(!TextUtils.isEmpty(callerappid))
			pushSessionInfo(SessPerflog.KEY_APP_CALLER_APPID, callerappid, false);
		
		HashParam appInfo = AppInfoUtil.getAppInfo(null);
		
		String version = appInfo.getString(AppInfoUtil.APP_VER_CODE);
		if(!TextUtils.isEmpty(version))
			pushSessionInfo(SessPerflog.KEY_APP_VER_CODE, version, false);
	}
	/**
	 * 记录一个时间点到key中
	 * @param key
	 */
	public synchronized void pushSessionInfo(String key) 
	{
		pushSessionInfo(key, (SystemClock.elapsedRealtime() - app_RelateStart), false);
	}
	
	/**
	 * 记录一个时间点到key中或者到key的JSONArray中
	 * @param key
	 * @param isArrayData
	 */
	public synchronized void pushSessionInfo(String key, boolean isArrayData) 
	{
		pushSessionInfo(key, (SystemClock.elapsedRealtime() - app_RelateStart), isArrayData);
	}
	/**
	 * 记录一个long值到key中或者到key的JSONArray中
	 * @param key
	 * @param value
	 * @param isArrayData
	 */
	public synchronized void pushSessionInfo(String key, long value, boolean isArrayData)
	{
		if(TextUtils.isEmpty(key))
			return;
		try {
			if(!isArrayData)
				mSessionInfo.put(key, value);
			else 
			{
				JSONArray array = mSessionInfo.optJSONArray(key);
				if(null == array) {
					array = new JSONArray();
					mSessionInfo.put(key, array);
				}
				
				if(null != array) {
					array.put(value);
//					mSessionInfo.put(key, array);
				}
			}
		} catch (JSONException e) {
			DebugLog.LogE(e);
		}
	}
		
		/**
		 * 记录一个String值到key中或者到key的JSONArray中
		 * @param key
		 * @param value
		 * @param isArrayData
		 */
		public synchronized void pushSessionInfo(String key, String value, boolean isArrayData)
		{
			if(TextUtils.isEmpty(key) || TextUtils.isEmpty(value))
				return;
			try {
				if(!isArrayData)
					mSessionInfo.put(key, value);
				else 
				{
					JSONArray array = mSessionInfo.optJSONArray(key);
					if(null == array) {
						array = new JSONArray();
						mSessionInfo.put(key, array);
					}
					
					if(null != array) {
						array.put(value);
//						mSessionInfo.put(key, array);
					}
				}
			} catch (JSONException e) {
				DebugLog.LogE(e);
			}
		}
		
		public synchronized String getSessionInfo()
		{
			return mSessionInfo.toString();
		}
	
	//性能信息上传key
	public static final String KEY_SESSIONINFO = "sessinfo";
	//启动识别的时间
	public final static String KEY_APP_START = "app_start";
	// 第一次获取到录音数据时间，示例：2013/1/25-17:38:53 679
	public static final String KEY_REC_START = "rec_start";
	//开始启动录音机时间
	public static final String KEY_REC_OPEN = "rec_open";
	//录音机启动成功时间
	public static final String KEY_REC_READY = "rec_ready";
	//是否主动停止，示例：1,0
	public static final String KEY_REC_USTOP = "rec_ustop";
	//应用调用MSC的SSB时间（相对于应用层基准时间）
	public static final String KEY_APP_SSB = "app_ssb";
	//应用层错误码
	public static final String KEY_APP_RET= "app_ret";
	//首次获取到语音写入MSC时间
	public static final String KEY_APP_FAU = "app_fau";
	//将最后一段音频写入MSC时间
	public static final String KEY_APP_LAU = "app_lau";
	//首结果时间
	public static final String KEY_APP_FRS= "app_frs";
	//尾结果获取时间
	public static final String KEY_APP_LRS= "app_lrs";
	//录音机关闭时间
	public static final String KEY_REC_CLOSE= "rec_close";
	//用户点击停止时间
	public static final String KEY_APP_STOP= "app_stop";
	//首结果展现时间
	public static final String KEY_UI_FRS= "ui_frs";
	//尾结果展现时间
	public static final String KEY_UI_LRS= "ui_lrs"; 
	//语记调用者APPID
	public static final String KEY_APP_CALLER_APPID= "app_caller_appid";
	//应用版本号
	public static final String KEY_APP_VER_CODE = "app_cver";
	//VAD输出第一块音频的语音位置(Bytes)
	public static final String KEY_VAD_VPOS = "vad_vpos";
	//前端点语音位置(Bytes)
	public static final String KEY_VAD_SPOS = "vad_spos";
	//后端点语音位置(Bytes)
	public static final String KEY_VAD_EPOS = "vad_epos";
	//应用层VAD开始时间
	public static final String KEY_VAD_FAU  = "vad_fau";
	//最后一条音频写入VAD时间
	public static final String KEY_VAD_LAU  = "vad_lau";
	//VAD首音频输出时间
	public static final String KEY_VAD_FOS  = "vad_fos";
	//VAD尾音频输出时间
	public static final String KEY_VAD_EOS  = "vad_eos";
	//VAD中间音频输出时间
	public static final String KEY_VAD_AUWT = "vad_auwt";
	//VAD中间音频输出时间
	public static final String KEY_VAD_AUWL = "vad_auwl";
}
