package com.iflytek.cloud.msc.msp;

import java.io.IOException;

import android.content.Context;
import android.os.HandlerThread;
import android.os.Message;
import android.text.TextUtils;

import com.iflytek.cloud.ErrorCode;
import com.iflytek.cloud.SpeechError;
import com.iflytek.cloud.SpeechListener;
import com.iflytek.cloud.SpeechUtility;
import com.iflytek.cloud.msc.module.MscHandler;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.param.HashParam;
import com.iflytek.cloud.param.MscKeys;

public class MscCommon extends MscHandler{

	private SpeechListener mListener = null; 
	private ComSession  mSession   = new ComSession();
	private String mSubType = null;

	public MscCommon(Context context,HashParam param,HandlerThread handlerthread) {
		super(context,handlerthread);
		setParams(param);
	}
	public MscCommon(Context context,HashParam param) {
		super(context);
		setParams(param);
	}

	/**
	 * 下载用户数据
	 * @param listener  监听器
	 */
	public void startDownload(SpeechListener listener) {
		mListener = listener;
		sendMsg( MSG_DOWNLOAD );
	}
	/**
	 * 语义搜索
	 * @param listener 监听器
	 * @param text 文本内容
	 */
	public void startUnderstandText(SpeechListener listener,String text) {
		setStatus(Status.start);
		mSubType = MscKeys.SUB_SCH;
		mListener = listener;
		
		/*try{
			this.mCollector.onSessionBegin();
		}catch( Throwable e ){
			DebugLog.LogE( "DC exception:" );
			DebugLog.LogE( e );
		}*/
		
		Message msg = obtainMessage( MSG_SEARCH, text );
		sendMsg( msg );
	}
	/**
	 * 上传
	 * @param listener
	 * @param name
	 * @param param
	 * @param uploaddata
	 */
	public void startUpload(SpeechListener listener,String name,byte[] uploaddata) {
		mSubType = MscKeys.SUB_UUP;
		mListener = listener;
		CommonInfo cominfo = new CommonInfo( uploaddata, name );
		Message msg = obtainMessage( MSG_UPLOAD, cominfo );
		sendMsg( msg );
	}

	/**
	 * 执行登陆操作
	 */
	public SpeechError login(String usr,String pwd)
	{
		mSubType = MscKeys.SUB_AUTH;
		SpeechError error = null;
		try
		{
			ComSession.Login(mContext,usr,pwd,this);
		}
		catch( SpeechError e )
		{
			DebugLog.LogE(e);
			error = e;
		}
		catch( IOException e )
		{
			DebugLog.LogE(e);
			error = new SpeechError(ErrorCode.ERROR_FILE_ACCESS);
		}
		catch( Exception e )
		{
			DebugLog.LogE(e);
			error = new SpeechError(ErrorCode.ERROR_ENGINE_INIT_FAIL);
		}
		finally{
			if(error != null)
			{
				DebugLog.LogD(getTag() + " occur Error = " + error.toString());
			}
		}
		return error;
	}



	@Override
	protected void onMsgProcess( Message msg ) throws Throwable, SpeechError
	{
		super.onMsgProcess( msg );
		
		if( null == SpeechUtility.getUtility() ){
			DebugLog.LogE( "MscCommon process while utility is null!" );
			exit( new SpeechError(ErrorCode.ERROR_LOGIN) );
			return;
		}
		
		byte[] mresult  = null;
		switch( msg.what )
		{
		case MSG_UPLOAD:
			CommonInfo cominfo = (CommonInfo)msg.obj;
			if(cominfo.getuploadData() == null || cominfo.getuploadData().length <= 0)
				throw new SpeechError(ErrorCode.ERROR_EMPTY_UTTERANCE);
			mresult = mSession.uploadData(mContext,cominfo.getdataName(),cominfo.getuploadData(),this);
			break;
		case MSG_DOWNLOAD:
			mresult = mSession.downLoadData(mContext,this);
			break;
		case MSG_SEARCH:
			String text = (String)msg.obj;
			if(TextUtils.isEmpty(text))
				throw new SpeechError(ErrorCode.ERROR_EMPTY_UTTERANCE);
			setStatus(Status.waitresult);
			mresult = mSession.searchResult(mContext,this,text);

			break;
		default:
			break;
		}
		if(mresult == null)
			throw new SpeechError(ErrorCode.ERROR_INVALID_RESULT);
		else if(mListener != null)
		{
			if(!mUserCancel) {
				mListener.onBufferReceived(mresult);
			}
		}
		exit( null );
	}
	@Override
	protected void onEnd(SpeechError error) {
		super.onEnd(error);
		if(mListener != null)
		{
			if(!mUserCancel)
				mListener.onCompleted(error);
			else
				;//mListener.onCancel(); 
		} 
	}
	/**
	 * 返回客户端ID，为NULL
	 */
	public String getClientID() {
		return null;
	}
	/**
	 * 参数传递对象
	 * @author hongyu
	 *
	 */
	private class CommonInfo{
		private byte[]     uploadData = null;
		private String      dataName    = "";
		/**
		 * 
		 * @param uploaddata 上传数据
		 * @param dataname	   上传名
		 */
		public CommonInfo(byte[] uploaddata,String dataname){
			this.uploadData = uploaddata;
			this.dataName = dataname;
		}
		/**
		 * 获取上传数据
		 * @return
		 */
		public byte[] getuploadData()
		{
			return uploadData;
		}
		/**
		 * 获取上传名
		 * @return
		 */
		public String getdataName()
		{
			return dataName;
		}

	}
	@Override
	public String getSessionID()
	{
		return null;
	}

}
