package com.iflytek.cloud.msc.tts;

import java.io.UnsupportedEncodingException;

import android.content.Context;
import android.text.TextUtils;

import com.iflytek.cloud.ErrorCode;
import com.iflytek.cloud.SpeechError;
import com.iflytek.cloud.msc.module.MscHandler;
import com.iflytek.cloud.msc.module.MscSession;
import com.iflytek.cloud.msc.util.DataUtil;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.msc.util.log.PerfLogger;
import com.iflytek.cloud.param.ParamBuilder;
import com.iflytek.msc.MSC;
import com.iflytek.msc.MSCSessionInfo;

public class TtsSession extends MscSession{


	// *********************************************************

	/** member properties */
	private MSCSessionInfo mOutParams = new MSCSessionInfo();

	@Override
	public int sessionBegin(Context context,
			String extra, MscHandler looper) throws SpeechError,
			UnsupportedEncodingException {
		/** reset internal state */
		mClientID = null;
		/** invoke native code */
		String separam = ParamBuilder.composeTtsSessionParam(context,looper);
		DebugLog.LogD("QTTSSessionBegin enter");
		long d1 = System.currentTimeMillis();
		byte[] padata = separam.getBytes(looper.getParamEncoding());
		 //加锁防止外部多线程造成同时ssb成功，之后回调造成的崩溃。
		PerfLogger.appendInfo(PerfLogger.MSC_SESSION_BIGNE, null);
        synchronized (TtsSession.class) {
        	mClientID = MSC.QTTSSessionBegin(padata, mOutParams);
        }
        PerfLogger.appendInfo(PerfLogger.SESSION_BEGIN_END, null);
		DebugLog.LogD("QTTSSessionBegin leave:" + (System.currentTimeMillis()-d1)
							+ " ErrorCode:" + mOutParams.errorcode);

		/** save last error and return */
		int errorCode = mOutParams.errorcode;
		if (0 != errorCode) {
			//增加防止10109、10132错误直接返回
			if(errorCode != ErrorCode.MSP_ERROR_CREATE_HANDLE
					&& errorCode != ErrorCode.MSP_ERROR_OVERFLOW && errorCode != ErrorCode.MSP_ERROR_INVALID_OPERATION)
				throw new SpeechError(errorCode);
		}
		return errorCode;
	}
	
	@Override
	public void sessionEnd(String reason) {
		/** test session */
		if (null == mClientID) {
			return;
		}
		if(TextUtils.isEmpty(reason))
			reason = "unknown";
		/** invoke native code */
		DebugLog.LogD("QTTSSessionEnd enter");
		int nRet = MSC.QTTSSessionEnd(mClientID, reason.getBytes());
		DebugLog.LogD("QTTSSessionEnd leavel:" + nRet);
		/** reset session */
		mClientID = null;
		mSessionID = null;
		return;
	}


	/**
	 * put content what you want to synthesize
	 * 
	 * @param data
	 *            content what you want to synthesize
	 * @return
	 */
	public synchronized void putText(byte[] data) throws SpeechError {

		/** invoke native code */
		DebugLog.LogD("QTTSTextPut enter");
		PerfLogger.appendInfo(PerfLogger.LAST_DATA_FLAG, null);
		int error = MSC.QTTSTextPut(mClientID, data);
		DebugLog.LogD("QTTSTextPut leavel:" + error);
 
		/** raise exception */
		if (0 != error) {
			throw new SpeechError(error);
		}
	}

	/**
	 * get audio data
	 * 
	 * @param buffer
	 *            storage for audio data
	 * @return
	 */
	public synchronized byte[] getAudio() throws SpeechError {
		byte[] buffer = null;
		if (null == mClientID)
			throw new SpeechError(ErrorCode.ERROR_NET_EXCEPTION);

		/** invoke native code */
		DebugLog.LogI("QTTSAudioGet enter");
		buffer = MSC.QTTSAudioGet(mClientID, mOutParams);
		DebugLog.LogI("QTTSAudioGet leave:" + mOutParams.errorcode
				+ "value len = " + (buffer == null ? 0 : buffer.length));

		int error = mOutParams.errorcode;
		/** raise exception */
		if (0 != error) {
				throw new SpeechError(error);
		}

		return buffer;
	}

	/**
	 * @return current audio synthesizer percent
	 */
	public int getAudioIndex() {
		int ret = 0;
		try{
			String ced = getStringValue("ced");
			ret = Integer.parseInt(ced);
			//DebugLog.LogE("ced:"+ret);
		}catch(Exception e)
		{
		}
		return ret;
	}
	
	/**
	 * @return current audio synthesizer spell information
	 */
	public String getAudioInfo()
	{
		String ret = "";
		try {
			char[] info = MSC.QTTSAudioInfo(mClientID);
			if( null != info && info.length>0 ){
				ret = new String(info);
			}
			
		} catch(Exception e) {
			DebugLog.LogE(e);
		}
		return ret;
	}
	
	/**
	 * get audio data volume value
	 * 
	 * @return
	 */
	synchronized public int getIntValue(String tag) 
	{
		if(mClientID == null)
			return 0;
		int ret = 0;
		try {
			String value = getStringValue(tag);
			if (!TextUtils.isEmpty(value)) {
				ret = Integer.parseInt(new String(value));
			} else {
			}
		} catch (Exception e) {
		}
		return ret;
	}
	
	/**
	 * get audio data volume value
	 * 
	 * @return
	 */
	synchronized public String getStringValue(String tag) {
		if(mClientID == null)
			return null;
		int code = 0;
		try {
			code = MSC.QTTSGetParam(mClientID, tag.getBytes(),mOutParams);
			if (code == 0) {
				return new String(mOutParams.buffer);
			} else {
			}
		} catch (Exception e) {
		}
		return null;
	}
	
	/**
	 * 获取Int类型变量
	 * @param tag
	 * @return
	 */
	public static int getIntByTag(String tag)
	{
		try {
			String val = getStringByTag(tag);
			if(TextUtils.isEmpty(val))
				return 0;
			return Integer.parseInt(val);
		} catch (Exception e) {
		}
		return 0;
	}
	
	/**
	 * 获得MSC String类型变量
	 * @param tag
	 * @return
	 */
	public static String getStringByTag(String tag)
	{
		try {
			int code = 0;
			MSCSessionInfo info = new MSCSessionInfo();
			code = MSC.QTTSGetParam(null, tag.getBytes(),info);
			if (code == 0) {
				return new String(info.buffer).trim();
			} else {
			}
		} catch (Exception e) {
		}
		return null;
	}

	/**
	 * test that previous data is last result
	 * 
	 * @return
	 */
	public synchronized boolean isLastResult() {
		return (2 == mOutParams.sesstatus);
	}
	/**
	 * 获得服务端ID
	 */
	protected String getSessionID()
	{
		if(null == mSessionID )
			mSessionID = getStringValue(TAG_SID);
		return mSessionID;
	}
}
