package com.iflytek.cloud.msc.util;

import java.io.UnsupportedEncodingException;
import java.nio.charset.Charset;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Random;

import org.json.JSONException;
import org.json.JSONObject;

import android.content.Context;
import android.text.TextUtils;
import android.util.DisplayMetrics;
import android.util.TypedValue;

import com.iflytek.cloud.msc.util.log.DebugLog;

public class DataUtil {
	
	public static final  int    SIZE_1K = 1024;
	public static final int SIZE_64K = 1024 * 64;
	public static final  String UTF8 = "utf-8";
	public static final  String GB2312 = "gb2312";
	public static final  String UNICODE = "unicode";
	public static final String DEF_CHARSET = GB2312;
	
	public static DisplayMetrics mMetrics = null;
	
	/**
	 * 返回字符串格式时间，精确到ms，示例：2013/1/25 17:38:53 679
	 * @param t 时间戳
	 * @return
	 */
	public static String getComplexDateTime(long t)
	{
		SimpleDateFormat sdf= new SimpleDateFormat("yyyy/MM/dd HH:mm:ss SSS");
		return sdf.format(t);
	}
	
	/**
	 * 返回字符串格式时间，精确到s，示例：2013-1-25 17:38:53
	 * @param t 时间戳
	 * @return
	 */
	public static String getSimpleDateTime(long t)
	{
		SimpleDateFormat sdf= new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		return sdf.format(t);
	}
	/**
	 * 返回字符串格式时间，用于插入到文件名当中作为时间戳，精确到ms，示例：20130125_173853100
	 * @param date 时间
	 * @return
	 */
	public static String getFileNameByDateTime(Date date)
	{
		SimpleDateFormat sdf= new SimpleDateFormat("yyyyMMdd_HHmmssSSS");
		try {
			return sdf.format(date);
		} catch (Exception e) {
			DebugLog.LogE(e);
		}
		return null;
	}
	
	public static int readInt(byte[] data,int offset)
	{
		int ret = 0;
		ret = data[offset+3] & 0xFF;
		ret |= ((data[offset+2] << 8) & 0xFF00);
		ret |= ((data[offset+1] << 16) & 0xFF0000);
		ret |= ((data[offset+0] << 24) & 0xFF000000); 
		return ret;
	}
	
	/**
	 * 将ascii数据转换成long型
	 * @param buffer
	 * @param offset
	 * @return
	 */
	public static long byteToLong(byte[] buffer,int offset)
	{
		if(buffer == null || buffer.length < 8)
			return 0;
		long ret = 0;
		
		for(int i = 0; i < 8; i++)
		{
			ret |= (buffer[i] & 0xFF);
			if(i < 7)
				ret <<= 8;
		}
		return ret;
	}
	
	/**
	 * 将long型转换成ascii值字符串，占八个字节
	 * @param val
	 * @return
	 * @throws UnsupportedEncodingException 
	 */
	public static byte[] longToByte(long val)
	{
		byte[] data = new byte[8];
		for(int i = 7; i >= 0; i--)
		{
			data[i] = (byte) (val & 0xFF);
			DebugLog.LogD(" i = " + data[i]);
			val >>= 8; 
		}
		return data;
	}
	
	/**
	 * 获取从begin 到 end 中间的随机随，半封闭区间，包括begin，不包括end
	 * @param begin
	 * @param end
	 * @return
	 */
	public static int RandomInt(int begin,int end)
	{
		int i = new Random().nextInt(end-begin);
		return begin + i;
	}
	
	public static int hexstring2Int(String hexstring){ 
           if(hexstring.substring(0,2).equalsIgnoreCase("0x"))
        	    hexstring = hexstring.substring(2);
        	hexstring = hexstring.toLowerCase();
           return (int)Long.parseLong(hexstring,16);
	}	
	
	public static int dip2px(Context context,float dpValue){
		if(mMetrics == null)
			mMetrics = context.getResources().getDisplayMetrics();
		 return	(int)TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, dpValue, mMetrics);
	}
	
	public static int sp2px(Context context,float spValue){
		if(mMetrics == null)
			mMetrics = context.getResources().getDisplayMetrics();
		return	(int)TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_SP, spValue, mMetrics);
	}
	
	/**
	 * 对sp的值进行校正，校正值=原值*metrics.density/metrics.scaledDensity
	 * @param context
	 * @param spValue
	 * @return
	 */
	public static int spModify(Context context,float spValue){
		if(mMetrics == null)
			mMetrics = context.getResources().getDisplayMetrics();
		float size = spValue
			*  TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 1, mMetrics)
			/ TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_SP, 1, mMetrics);
		return	(int)size;
	}
	
	/**
	 * 将字节缓冲区按照固定大小进行分割成数组
	 * @param buffer 缓冲区
	 * @param length 缓冲区大小
	 * @param spsize 切割块大小
	 * @return
	 */
	public static ArrayList<byte[]> splitBuffer(byte[] buffer,int offset,int length,int spsize)
	{
		ArrayList<byte[]> array = new ArrayList<byte[]>();
		if(spsize <= 0 || length <= 0 || buffer == null || buffer.length < (length+offset))
			return array;
		int size = 0;
		while(size < (length-offset))
		{
			int left = length - size - offset;
			if(spsize < left)
			{
				byte[] sdata = new byte[spsize];
				System.arraycopy(buffer,size+offset,sdata,0,spsize);
				array.add(sdata);
				size += spsize;
			}else
			{
				byte[] sdata = new byte[left];
				System.arraycopy(buffer,size+offset,sdata,0,left);
				array.add(sdata);
				size += left;
			}
		}
		return array;
	}
	
	/**
	 * 为二进制文本加上utf-8的bom头
	 * 
	 * @param text utf-8编码的二进制文本
	 * @return 带bom头的二制文本
	 */
	public static byte[] getUTF8Bom(byte[] text) {
		byte[] buffer = new byte[3 + text.length];
		buffer[0] = (byte) 0xEF;
		buffer[1] = (byte) 0xBB;
		buffer[2] = (byte) 0xBF;
		System.arraycopy(text, 0, buffer, 3, text.length);

		return buffer;
	}
	
	/**
	 * 为字符串加上utf-8的bom头
	 * 
	 * @param text utf-8编码的字符串
	 * @return 带bom头的二制文本
	 */
	public static byte[] getUTF8Bom(String text) {
		byte[] buffer = null;
        try {
        	byte[] textBytes = text.getBytes(DataUtil.UTF8);
        	buffer = new byte[3 + textBytes.length];
        	
            buffer[0] = (byte)0xEF;
            buffer[1] = (byte)0xBB;
            buffer[2] = (byte)0xBF;
        	
        	for (int i = 3; i < buffer.length; i++) {
    			buffer[i] = textBytes[i - 3];
    		}
		} catch (UnsupportedEncodingException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
    
	    return buffer;
	}
	
	/**
	 * 
	 * @param key
	 * @param obj1
	 * @param obj2
	 * @return
	 */
	public static boolean compareJsonInt(String key, JSONObject obj1, JSONObject obj2) {
		try {
			int value1 = obj1.getInt(key);
			int value2 = obj2.getInt(key);
			if(value2 < value1) {
				return false;
			}
		} catch (JSONException e) {
			DebugLog.LogE("invalid jsonobject,compare "+ key + " error");
			return false;
		}
		return true;
	}
	
	/**
	 * 
	 * @param key
	 * @param obj1
	 * @param obj2
	 * @return
	 */
	public static boolean compareJsonString(String key, JSONObject obj1, JSONObject obj2) {
		try {
			String value1 = obj1.getString(key);
			String value2 = obj2.getString(key);
			if (TextUtils.isEmpty(value1) || TextUtils.isEmpty(value2)
					|| !value1.equalsIgnoreCase(value2)) {
				return false;
			}
		} catch (JSONException e) {
			DebugLog.LogE("invalid jsonobject,compare "+ key + " error");
			return false;
		}
		return true;
	}
	
	/**
	 * 
	 * @param key
	 * @param obj1
	 * @param obj2
	 * @return
	 */
	public static boolean compareJsonLong(String key, JSONObject obj1, JSONObject obj2) {
		try {
			long value1 = obj1.getLong(key);
			long value2 = obj2.getLong(key);
			if (value2 < value1) {
				return false;
			}
		} catch (JSONException e) {
			DebugLog.LogE("invalid jsonobject,compare "+ key + " error");
			return false;
		}
		return true;
	}
	
	public static byte[] getNativeByteArray( final String srcString, final String dstCharset ) throws UnsupportedEncodingException{
		byte[] dstBytes = null;
		
		if( null != srcString ){
			if( 0 < srcString.length() ){
				final byte[] tmpBytes = srcString.getBytes( dstCharset );
				dstBytes = new byte[ tmpBytes.length+1 ];
				System.arraycopy( tmpBytes, 0, dstBytes, 0, tmpBytes.length );
				dstBytes[tmpBytes.length] = 0;
			}else{
				dstBytes = new byte[1];
				dstBytes[0] = 0;
			}//end of if-else srcString length > 0
		}//end of if-else srcString not null
		
		return dstBytes;
	}
	
	public static byte[] getNativeByteArray( final String srcString ) throws UnsupportedEncodingException{
		return getNativeByteArray( srcString, DEF_CHARSET );
	}
	
	public static String getStringFromNativeByteArray( final byte[] srcBytes, final String srcCharset ){
		String dstValue = null;
		
		if( null != srcBytes ){
			int i = 0;
			for( i=0; i<srcBytes.length; ++i ){
				if( 0 == srcBytes[i] ){
					break;
				}
			}//end of for
			
			int len = Math.min( i, srcBytes.length );
			dstValue = new String( srcBytes, 0, len, Charset.forName(srcCharset) );
		}
		
		return dstValue;
	}
	
	public static String getStringFromNativeByteArray( final byte[] srcBytes ){
		return getStringFromNativeByteArray( srcBytes, DEF_CHARSET );
	}
}
