package com.iflytek.cloud.msc.util;

import java.io.BufferedInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;

//import org.apache.http.util.ByteArrayBuffer;
import org.json.JSONObject;

import android.text.TextUtils;
import android.util.Base64;

import com.iflytek.cloud.ErrorCode;
import com.iflytek.cloud.SpeechError;
import com.iflytek.cloud.msc.util.log.DebugLog;

/**
 * http网络请求类，封装http请求
 * 此对象主要是请求较小资源的数据，不适用于大块数据的下载
 * @author yjzhao
 *
 */
public class HttpRequest extends Thread{

	public interface HttpRequestListener
	{
		public void onResult(HttpRequest request,byte[] data);
		public void onError(SpeechError e);
	}

	private int     mTimeOut  = 20000; // 20s 
	// 回调消息
	private HttpRequestListener mListener = null;
	// 是否终止
	private volatile boolean mExit = false;
	// 请求url
	private URL  mUrl = null;
	// Post数据
	private ArrayList<byte[]>   postDatas = new ArrayList<byte[]>();
	// Post头部参数
	private String mUpLoadParam;

	// http线程保存的临时数据，用以上传错误时数据回写
	private Object httpbuf = null;

	public static final int CONNECT_GET = 0;
	public static final int CONNECT_POST = 1;
	// Http请求类型，默认为Get方式
	private int mConnectType = CONNECT_GET;

	public HttpRequest() {

	}
	/**
	 * 
	 * @param url 请求Url
	 * @param param url参数
	 * @param data Post数据
	 */
	public HttpRequest(String url,String param,byte[] data)
	{
		setRequest(url,param,data);
	}

	/**
	 * 临时数据
	 * @param httpbuf
	 */
	public void setHttpbuf(Object httpbuf) {
		this.httpbuf = httpbuf;
	}
	public Object getHttpbuf() {
		return httpbuf;
	}

	public void setConectType(int type)
	{
		mConnectType = type;
	}

	/**
	 * 设置请求数据
	 * @param url   请求url
	 * @param param url参数
	 * @param data  Post数据
	 */
	public void setRequest(String url,String param,byte[] data)
	{
		postDatas.clear();
		appendData(data);
		try {
			mUrl = encodeUrl(url, param);
		} catch (MalformedURLException e) {
			DebugLog.LogE(e);
		}
	}

	/**
	 * 设置请求数据
	 * @param url   请求url
	 * @param param url参数
	 * @param data  Post数据
	 * @param upLoadParam 上传的请求参数
	 */
	public void setRequest(String url,String param,byte[] data, String upLoadParam)
	{
		mUpLoadParam = upLoadParam; 
		setRequest(url,param,data);
	}
	
	/**
	 * 设置超时时间
	 * @param timeout
	 */
	public void setTimeOut(int timeout)
	{
		mTimeOut = timeout;
	}
	/**
	 * 开始http请求，启动线程异步处理
	 * @param listener
	 */
	public void startRequest(HttpRequestListener listener)
	{
		mListener = listener;
		start();
	}

	/**
	 * 将参数追加到url
	 * @param baseurl
	 * @param param
	 * @return
	 * @throws MalformedURLException
	 * @throws UnsupportedEncodingException
	 */
	public static URL encodeUrl(String baseurl,String param) throws MalformedURLException
	{
		String url = baseurl;
		if(!TextUtils.isEmpty(baseurl) && !TextUtils.isEmpty(param))
		{
			if(!baseurl.endsWith("?"))
				url += "?";
			url += param;
		}
		return (new URL(url));
	}

	public void cancel()
	{
		mExit = true;
	}

	@Override
	public void run() {
		if(mConnectType == CONNECT_POST)
			runPost();
		else
			runGet();
	}

	/**
	 * 读取数据流
	 * @param in http流
	 * @return 数据
	 * @throws IOException
	 */
	private byte[] readStream(InputStream in) throws IOException
	{
		int ONE_K = 1024;
		BufferedInputStream bis = new BufferedInputStream(in);
		//ByteArrayBuffer buf = new ByteArrayBuffer(ONE_K);
		ByteArrayOutputStream buf = new ByteArrayOutputStream() ;
		int templen = 0;
		byte[] temp = new byte[ONE_K];
		while (!mExit && (templen = bis.read(temp,0,ONE_K)) != -1) {
			//buf.append(temp,0,templen);
			buf.write(temp,0,templen);
		}
		return buf.toByteArray();
	}
	private void runGet()
	{
		HttpURLConnection urlConn = null;
		InputStream in = null;

		try {
			DebugLog.LogD("Start connect server");
			urlConn = (HttpURLConnection) mUrl.openConnection();
			urlConn.setConnectTimeout(mTimeOut);
			urlConn.setReadTimeout(mTimeOut);
			urlConn.setRequestMethod("GET");

			int responseCode = urlConn.getResponseCode();
			DebugLog.LogD("responseCode = " + responseCode);
			if(HttpURLConnection.HTTP_OK == responseCode)
			{
				in = urlConn.getInputStream();
				throwResult(readStream(in));
			}else{
				DebugLog.LogD("MscHttpRequest connect error:"+responseCode);
				//HTTP错误基址+code
				throwError(new SpeechError(ErrorCode.MSP_ERROR_HTTP_BASE+Math.abs(responseCode)));
			}
		} catch (Exception e) {
			DebugLog.LogE(e);
			DebugLog.LogD("MscHttpRequest error:" + e.toString());
			throwError(new SpeechError(ErrorCode.ERROR_NET_EXCEPTION));
		}finally
		{
			try {
				if(in != null)
				{
					in.close();
					in = null;
				}
				if(urlConn != null){
					urlConn.disconnect();
					urlConn = null;
				}
			} catch (Exception e) {
			}
		}
	}

	/**
	 * 通知回调结果
	 * @param data
	 */
	private void throwResult(byte[] data)
	{
		if(mListener == null || mExit)
			return;
		mListener.onResult(this,data);
	}

	/**
	 * 追加数据
	 * @param data
	 */
	public void appendData(byte[] data) {
		if(data != null)
			postDatas.add(data);
	}
	/**
	 * 请求发生错误
	 * @param e
	 */
	private void throwError(SpeechError e)
	{
		if(mListener == null || mExit)
			return;
		mListener.onError(e);
	}

	private int getPostLength()
	{
		int ret = 0;
		for(int i = 0; i < postDatas.size(); i++)
		{
			ret += postDatas.get(i).length;
		}
		return ret;
	}

	/**
	 * 开始Post请求
	 */
	public void runPost(){

		HttpURLConnection httpConn = null;		
		InputStream in = null;
		try{
			DebugLog.LogD("MscHttpRequest start Post");
			httpConn =(HttpURLConnection)mUrl.openConnection();
			httpConn.setDoOutput(true);
			httpConn.setDoInput(true);
			httpConn.setUseCaches(false);
			httpConn.setRequestMethod("POST");
			httpConn.setConnectTimeout(mTimeOut);
			httpConn.setReadTimeout(mTimeOut);
			httpConn.setRequestProperty("Content-length", "" + getPostLength());
			//httpConn.setRequestProperty("Content-Type", "application/octet-stream");
			httpConn.setRequestProperty("Connection", "Keep-Alive");
			httpConn.setRequestProperty("Charset", DataUtil.UTF8);
			httpConn.setRequestProperty("Content-Type", "application/x-gzip");
			if (!TextUtils.isEmpty(mUpLoadParam)) {
				//Base64不加换行符
				String base = Base64.encodeToString(mUpLoadParam.getBytes(), Base64.NO_WRAP);
				httpConn.setRequestProperty("X-Par",base);
			}
			OutputStream outputStream = httpConn.getOutputStream();
			for(byte[] buf : postDatas)
			{
				//DebugLog.LogD("MscHttpRequest write length:" + buf.length);
				outputStream.write(buf);  
			}
			outputStream.flush();
			outputStream.close();
			
			int responseCode = httpConn.getResponseCode();
			// DebugLog.LogD("responseCode = " + responseCode);
			if(HttpURLConnection.HTTP_OK == responseCode){
				in = httpConn.getInputStream();
				throwResult(readStream(in));
			}
			else{
				DebugLog.LogD("Http Request Failed."+responseCode);
				//HTTP错误基址+code
				throwError(new SpeechError(ErrorCode.MSP_ERROR_HTTP_BASE+responseCode));
			}

		}catch(Exception e){
			throwError(new SpeechError(ErrorCode.ERROR_NET_EXCEPTION));
			DebugLog.LogD("MscHttpRequest error:" + e.toString());
		}finally{
			try {
				if(in != null){
					in.close();
					in =null;
				}
				if(httpConn != null){
					httpConn.disconnect();
					httpConn =null;
				}
			} catch (Exception e) {
			}
		}
	}
	
	/**
	 * 检测是否发生错误
	 * @return
	 */
	public static boolean hasHttpError(String buffer)
	{
		try {
			JSONObject object = new JSONObject(buffer);
			if(object.has("rsp"))
			{
				JSONObject error = object.getJSONObject("rsp");
				int code = error.getInt("code");
				if(code != 0)
				{
//					String descript = error.getString("des");
					return true;
				}
			}
		} catch (Exception e) {
		}
		return false;
	}
}