package com.iflytek.cloud.ui.control;

import android.content.Context;
import android.content.Intent;
import android.util.AttributeSet;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;

import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.ui.control.MscDialog.DismissCallBack;

public class MscView extends LinearLayout{

	private DismissCallBack mDismissCall = null;
	
	// 是否显示错误页面
	protected boolean mShow_error_view = true;

	// 是否显示错误码
	protected boolean mShow_error_code = true;
	
	public MscView(Context context, AttributeSet attrs) {
		super(context, attrs);
	}
	
	public MscView(Context context) {
		super(context);
	}
	
	/**
	 * 是否显示错误页面和错误码，默认均显示
	 * @param showView 是否显示错误界面
	 * @param showCode 是否在错误界面显示错误码
	 */
	public void showErrorView(boolean showView,boolean showCode)
	{
		mShow_error_view = showView;
		mShow_error_code = showCode;
	}
	
	protected void openWireLessView()
	{
		Intent intent = new Intent();
		if(android.os.Build.VERSION.SDK_INT > 10 ){
	         //3.0以上打开设置界面
			intent.setAction(android.provider.Settings.ACTION_SETTINGS);
		}else
		{
			intent.setAction(android.provider.Settings.ACTION_WIRELESS_SETTINGS);
		}
		intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
		getContext().startActivity(intent);
	}
	
	/**
	 * 设置对话框关闭事件回调
	 * @param dismissCall
	 */
	public void setExitCallBack(DismissCallBack dismissCall)
	{
		mDismissCall = dismissCall;
	}
		
	/**
	 * 关闭接口
	 */
	public void dismiss()
	{
		if(mDismissCall != null)
			mDismissCall.exit();
	}
	
	/**
	 * 启动
	 */
	protected void start() {
	}
	
	/**
	 * 退出
	 */
	protected void cancel()
	{
	}
	
	/**
	 * 释放View对象
	 * @param view
	 */
	protected static boolean uninitView(ViewGroup view)
	{
		try {
			int clcount = view.getChildCount();
			for(int i = 0; i < clcount; i++)
			{
				View v = view.getChildAt(i);
				v.setOnClickListener(null); 
				v.setBackgroundDrawable(null);
			}
			view.removeAllViews();
			view.setOnClickListener(null);
			view.setBackgroundDrawable(null);
			return true;
		} catch (Exception e) {
			DebugLog.LogE(e);
		}
		return false;
	}
	
	/**
	 * 销毁View对象
	 */
	protected boolean destroy()
	{
		try {
			uninitView(this);
			return true;
		} catch (Exception e) {
			DebugLog.LogE(e);
		}
		return false;
	}
}
