package com.iflytek.cloud.util.code.impl;

public abstract class Codec {
	/************************************************************/
	/**
	 * @summary	实例信息，保存错误码和实例句柄
	 * encodeHandle 编码句柄，外部可不理会此成员，外部不可更改该成员值；
	 * decodeHandle 解码句柄，外部可不理会此成员，外部不可更改该成员值；
	 * errorCode 错误码，当调用结果后，成功时为0，其它为失败；
	 * @author gxlin
	 */
	public static class InstanceInfo{
		public long encodeHandle = 0;
		public long decodeHandle = 0;
		public int errorCode = 0;
	}
	/************************************************************/
	
	/************************************************************/
	/**
	 * @summary AudioData类，用于编、解码时，保存结果音频的对象类
	 * bufLen 初始化时，构造的数组大小；调用结束时，该值会被改变，为实际音频缓存大小
	 * dataBuf 保存音频缓存的数组
	 * @author gxlin
	 */
	public static class AudioData{
		public int dataLen;
		public byte[] dataBuf;
		
		public AudioData( final int bufferLen ){
			dataLen = 0;
			dataBuf = new byte[bufferLen];
		}
		
		public void reset(){
			dataLen = 0;
		}
	}//	end of class AudioData
	/************************************************************/
	
	/************************************************************/
	public static final int DEF_MODE = 0;	//默认模式
	public static final int DEF_LEVEL = 0;	//默认等级
	/************************************************************/
	
	/************************************************************/
	/**
	 * @summary 初始化编码
	 * @param instInfo 请传入类InstanceInfo对象，返回时，检查该对象的errorCode成员值，instInfo.errorCode＝＝0时，调用成功，否则调用失败；
	 * @param encodeMode 相应的编码模式0或1（不同的库中可能代码不同的值）
	 * @return 错误码，与instInfo.errorCode相同
	 */
	public abstract int encodeInit( InstanceInfo instInfo, short encodeMode );
	
	/**
	 * @summary 逆初始化编码
	 * @param instInfo 请传入类InstanceInfo对象，返回时，检查该对象的errorCode成员值，instInfo.errorCode＝＝0时，调用成功，否则调用失败；
	 * @return 错误码，与instInfo.errorCode相同
	 */
	public abstract int encodeFini( InstanceInfo instInfo );
	
	/**
	 * @summary 编码
	 * @param instInfo 请传入类InstanceInfo对象，返回时，检查该对象的errorCode成员值，instInfo.errorCode＝＝0时，调用成功，否则调用失败；
	 * @param rawBuf 原始的音频数据
	 * @param rawBufLen 原始的音频数据大小
	 * @param audioData AudioData类对象，传入时，audioData.bufLen成员，用于指定当audioData.dataBuf成员的数组大小;
	 *        调用成功时，音频保存在audioData.dataBuf中，audioData.bufLen则用于指定实际音频缓存大小；
	 * @param mode 编码模式，speex时为0~10， am***的为0~7
	 * @return 错误码，与instInfo.errorCode相同
	 */
	public abstract int encode( InstanceInfo instInfo, byte[] rawBuf, int rawBufLen, AudioData audioData, short mode );
	
	/**
	 * @summary 初始化解码
	 * @param instInfo 请传入类InstanceInfo对象，返回时，检查该对象的errorCode成员值，instInfo.errorCode＝＝0时，调用成功，否则调用失败；
	 * @param decodeMode 当库为speex时，传入0或1，其它请输入0；
	 * @return 错误码，与instInfo.errorCode相同
	 */
	public abstract int decodeInit( InstanceInfo instInfo, short decodeMode );
	
	/**
	 * @summary 逆初始化解码
	 * @param instInfo 请传入类InstanceInfo对象，返回时，检查该对象的errorCode成员值，instInfo.errorCode＝＝0时，调用成功，否则调用失败；
	 * @return 错误码，与instInfo.errorCode相同
	 */
	public abstract int decodeFini( InstanceInfo instInfo );
	
	/**
	 * @summary 解码
	 * @param instInfo 请传入类InstanceInfo对象，返回时，检查该对象的errorCode成员值，instInfo.errorCode＝＝0时，调用成功，否则调用失败；
	 * @param encodeBuf 已编码的音频数据
	 * @param encodeBufLen 已编码的音频数据大小
	 * @param audioData AudioData类对象，传入时，audioData.bufLen成员，用于指定当audioData.dataBuf成员的数组大小;
	 *        调用成功时，音频保存在audioData.dataBuf中，audioData.bufLen则用于指定实际音频缓存大小；
	 * @return 错误码，与instInfo.errorCode相同
	 */
	public abstract int decode( InstanceInfo instInfo, byte[] encodeBuf, int encodeBufLen, AudioData audioData );
}